@@ -1,31 +1,17 @@
 /*
  * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -50,7 +36,7 @@
  * functions.
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "state.h"
 #include "conv.h"
 
@@ -144,7 +130,7 @@ conv_getctx(struct statelist *slist, struct ev *ev, unsigned hi, unsigned lo)
  * filled and 1 is returned.
  */
 unsigned
-conv_packev(struct statelist *l, unsigned xctlset,
+conv_packev(struct statelist *l, unsigned xctlset, unsigned flags,
 	    struct ev *ev, struct ev *rev)
 {
 	unsigned num, val;
@@ -153,89 +139,100 @@ conv_packev(struct statelist *l, unsigned xctlset,
 		rev->cmd = EV_XPC;
 		rev->dev = ev->dev;
 		rev->ch = ev->ch;
-		rev->pc_prog = ev->pc_prog;
-		rev->pc_bank = conv_getctx(l, ev, BANK_HI, BANK_LO);
+		rev->pc_prog = ev->v0;
+		rev->pc_bank = (flags & CONV_XPC) ?
+		    conv_getctx(l, ev, BANK_HI, BANK_LO) : 0;
 		return 1;
 	} else if (ev->cmd == EV_CTL) {
 		switch (ev->ctl_num) {
 		case BANK_HI:
+			if (!(flags & CONV_XPC))
+				break;
 			conv_rmctl(l, ev, BANK_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case RPN_HI:
+			if (!(flags & CONV_XPC))
+				break;
 			conv_rmctl(l, ev, NRPN_LO);
 			conv_rmctl(l, ev, RPN_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case NRPN_HI:
+			if (!(flags & CONV_NRPN))
+				break;
 			conv_rmctl(l, ev, RPN_LO);
 			conv_rmctl(l, ev, NRPN_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case DATAENT_HI:
+			if (!(flags & (CONV_RPN | CONV_NRPN)))
+				break;
 			conv_rmctl(l, ev, DATAENT_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case BANK_LO:
+			if (!(flags & CONV_XPC))
+				break;
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case NRPN_LO:
+			if (!(flags & CONV_NRPN))
+				break;
 			conv_rmctl(l, ev, RPN_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case RPN_LO:
+			if (!(flags & CONV_RPN))
+				break;
 			conv_rmctl(l, ev, NRPN_LO);
 			conv_setctl(l, ev);
-			break;
+			return 0;
 		case DATAENT_LO:
+			if (!(flags & (CONV_RPN | CONV_NRPN)))
+				break;
 			num = conv_getctx(l, ev, NRPN_HI, NRPN_LO);
 			if (num != EV_UNDEF) {
 				rev->cmd = EV_NRPN;
 			} else {
-				num = conv_getctx(l, ev, RPN_HI, NRPN_LO);
-				if (num == EV_UNDEF) {
-					break;
-				}
+				num = conv_getctx(l, ev,
+				    RPN_HI, NRPN_LO);
+				if (num == EV_UNDEF)
+					return 0;
 				rev->cmd = EV_RPN;
 			}
 			val = conv_getctl(l, ev, DATAENT_HI);
-			if (val == EV_UNDEF) {
-				break;
-			}
+			if (val == EV_UNDEF)
+				return 0;
 			rev->dev = ev->dev;
 			rev->ch = ev->ch;
 			rev->ctl_num = num;
 			rev->ctl_val = ev->ctl_val + (val << 7);
 			return 1;
-		default:
-			if (ev->ctl_num < 32) {
-				if (EVCTL_ISFINE(xctlset, ev->ctl_num)) {
-					conv_setctl(l, ev);
-					break;
-				}
-				rev->ctl_num = ev->ctl_num;
-				rev->ctl_val = ev->ctl_val << 7;
-			} else if (ev->ctl_num < 64) {
-				num = ev->ctl_num - 32;
-				if (!EVCTL_ISFINE(xctlset, num)) {
-					break;
-				}
+		}
+		if (ev->ctl_num < 32) {
+			if (EVCTL_ISFINE(xctlset, ev->ctl_num)) {
+				conv_setctl(l, ev);
+				return 0;
+			}
+		} else if (ev->ctl_num < 64) {
+			num = ev->ctl_num - 32;
+			if (EVCTL_ISFINE(xctlset, num)) {
 				val = conv_getctl(l, ev, num);
-				if (val == EV_UNDEF) {
-					break;
-				}
+				if (val == EV_UNDEF)
+					return 0;
 				rev->ctl_num = num;
 				rev->ctl_val = ev->ctl_val + (val << 7);
-			} else {
-				rev->ctl_num = ev->ctl_num;
-				rev->ctl_val = ev->ctl_val << 7;
+				goto done;
 			}
-			rev->cmd = EV_XCTL;
-			rev->dev = ev->dev;
-			rev->ch = ev->ch;
-			return 1;
 		}
-		return 0;
+		rev->ctl_num = ev->ctl_num;
+		rev->ctl_val = ev->ctl_val << 7;
+	done:
+		rev->cmd = EV_XCTL;
+		rev->dev = ev->dev;
+		rev->ch = ev->ch;
+		return 1;
 	} else {
 		*rev = *ev;
 		return 1;
@@ -248,13 +245,35 @@ conv_packev(struct statelist *l, unsigned xctlset,
  * the array.
  */
 unsigned
-conv_unpackev(struct statelist *slist, unsigned xctlset,
+conv_unpackev(struct statelist *slist, unsigned xctlset, unsigned flags,
 	      struct ev *ev, struct ev *rev)
 {
 	unsigned val, hi;
 	unsigned nev = 0;
 
 	if (ev->cmd == EV_XCTL) {
+		switch (ev->ctl_num) {
+		case BANK_HI:
+		case BANK_LO:
+			if (flags & CONV_XPC)
+				return 0;
+			break;
+		case NRPN_HI:
+		case NRPN_LO:
+			if (flags & CONV_NRPN)
+				return 0;
+			break;
+		case RPN_HI:
+		case RPN_LO:
+			if (flags & CONV_RPN)
+				return 0;
+			break;
+		case DATAENT_HI:
+		case DATAENT_LO:
+			if (flags & (CONV_NRPN | CONV_RPN))
+				return 0;
+			break;
+		}
 		if (ev->ctl_num < 32 && EVCTL_ISFINE(xctlset, ev->ctl_num)) {
 			hi = ev->ctl_val >> 7;
 			val = conv_getctl(slist, ev, ev->ctl_num);
@@ -285,33 +304,37 @@ conv_unpackev(struct statelist *slist, unsigned xctlset,
 			return 1;
 		}
 	} else if (ev->cmd == EV_XPC) {
-		val = conv_getctx(slist, ev, BANK_HI, BANK_LO);
-		if (val != ev->pc_bank && ev->pc_bank != EV_UNDEF) {
-			rev->cmd = EV_CTL;
-			rev->dev = ev->dev;
-			rev->ch = ev->ch;
-			rev->ctl_num = BANK_HI;
-			rev->ctl_val = ev->pc_bank >> 7;
-			conv_setctl(slist, rev);
-			rev++;
-			nev++;
-			rev->cmd = EV_CTL;
-			rev->dev = ev->dev;
-			rev->ch = ev->ch;
-			rev->ctl_num = BANK_LO;
-			rev->ctl_val = ev->pc_bank & 0x7f;
-			conv_setctl(slist, rev);
-			rev++;
-			nev++;
+		if (flags & CONV_XPC) {
+			val = conv_getctx(slist, ev, BANK_HI, BANK_LO);
+			if (val != ev->pc_bank && ev->pc_bank != EV_UNDEF) {
+				rev->cmd = EV_CTL;
+				rev->dev = ev->dev;
+				rev->ch = ev->ch;
+				rev->ctl_num = BANK_HI;
+				rev->ctl_val = ev->pc_bank >> 7;
+				conv_setctl(slist, rev);
+				rev++;
+				nev++;
+				rev->cmd = EV_CTL;
+				rev->dev = ev->dev;
+				rev->ch = ev->ch;
+				rev->ctl_num = BANK_LO;
+				rev->ctl_val = ev->pc_bank & 0x7f;
+				conv_setctl(slist, rev);
+				rev++;
+				nev++;
+			}
 		}
 		rev->cmd = EV_PC;
 		rev->dev = ev->dev;
 		rev->ch = ev->ch;
-		rev->pc_prog = ev->pc_prog;
+		rev->v0 = ev->pc_prog;
 		rev++;
 		nev++;
 		return nev;
 	} else if (ev->cmd == EV_NRPN) {
+		if (!(flags & CONV_NRPN))
+			return 0;
 		val = conv_getctx(slist, ev, NRPN_HI, NRPN_LO);
 		if (val != ev->rpn_num) {
 			conv_rmctl(slist, ev, RPN_HI);
@@ -350,6 +373,8 @@ conv_unpackev(struct statelist *slist, unsigned xctlset,
 		nev++;
 		return nev;
 	} else if (ev->cmd == EV_RPN) {
+		if (!(flags & CONV_RPN))
+			return 0;
 		val = conv_getctx(slist, ev, RPN_HI, RPN_LO);
 		if (val != ev->rpn_num) {
 			conv_rmctl(slist, ev, NRPN_HI);
