@@ -1,31 +1,17 @@
 /*
- * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
+ * Copyright (c) 2003-2019 Alexandre Ratchov <alex@caoua.org>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -162,9 +148,10 @@
  *	  the same as the event we write
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "track.h"
 #include "defs.h"
+#include "filt.h"
 #include "frame.h"
 #include "pool.h"
 
@@ -193,6 +180,7 @@ seqptr_new(struct track *t)
 
 	sp = (struct seqptr *)pool_new(&seqptr_pool);
 	statelist_init(&sp->statelist);
+	sp->link = NULL;
 	sp->pos = t->first;
 	sp->delta = 0;
 	sp->tic = 0;
@@ -205,10 +193,37 @@ seqptr_new(struct track *t)
 void
 seqptr_del(struct seqptr *sp)
 {
+	if (sp->link != NULL)
+		sp->link->link = NULL;
+
 	statelist_done(&sp->statelist);
 	pool_del(&seqptr_pool, sp);
 }
 
+/*
+ * return true if the end of the track is reached
+ */
+int
+seqptr_eot(struct seqptr *sp)
+{
+	return sp->delta == sp->pos->delta && sp->pos->ev.cmd == EV_NULL;
+}
+
+/*
+ * link a reader and a writer so that reader sees consistently
+ * changes by the writer
+ */
+void
+seqptr_link(struct seqptr *sp1, struct seqptr *sp2)
+{
+	if (sp1->link != NULL || sp2->link != NULL) {
+		log_puts("seqptr structures already linked\n");
+		panic();
+	}
+	sp1->link = sp2;
+	sp2->link = sp1;
+}
+
 /*
  * return the state structure of the next available event or NULL if
  * there is no next event in the current tick.  The state list is
@@ -223,7 +238,7 @@ seqptr_evget(struct seqptr *sp)
 		return 0;
 	}
 	st = statelist_update(&sp->statelist, &sp->pos->ev);
-	if (st->phase & EV_PHASE_FIRST) {
+	if (st->flags & STATE_NEW) {
 		st->pos = sp->pos;
 		st->tic = sp->tic;
 	}
@@ -268,17 +283,25 @@ seqptr_evdel(struct seqptr *sp, struct statelist *slist)
 struct state *
 seqptr_evput(struct seqptr *sp, struct ev *ev)
 {
+	struct seqptr *link;
 	struct seqev *se;
 
 	se = seqev_new();
 	se->ev = *ev;
 	se->delta = sp->delta;
 	sp->pos->delta -= sp->delta;
+
 	/* link to the list */
 	se->next = sp->pos;
 	se->prev = sp->pos->prev;
 	*se->prev = se;
 	sp->pos->prev = &se->next;
+
+	/* if there's a reader update its pointer */
+	link = sp->link;
+	if (link != NULL && link->pos == sp->pos)
+		link->pos = se;
+
 	/* fix the position pointer and update the state */
 	sp->pos = se;
 	return seqptr_evget(sp);
@@ -313,6 +336,7 @@ seqptr_ticskip(struct seqptr *sp, unsigned max)
 unsigned
 seqptr_ticdel(struct seqptr *sp, unsigned max, struct statelist *slist)
 {
+	struct seqptr *link;
 	unsigned ntics;
 
 	ntics = sp->pos->delta - sp->delta;
@@ -323,6 +347,14 @@ seqptr_ticdel(struct seqptr *sp, unsigned max, struct statelist *slist)
 	if (slist != NULL && max > 0) {
 		statelist_outdate(slist);
 	}
+
+	/* shift writer if affected */
+	link = sp->link;
+	if (link != NULL && link->pos == sp->pos && link->delta >= sp->delta) {
+		sp->link->delta -= ntics;
+		sp->link->tic -= ntics;
+	}
+
 	return ntics;
 }
 
@@ -332,11 +364,21 @@ seqptr_ticdel(struct seqptr *sp, unsigned max, struct statelist *slist)
 void
 seqptr_ticput(struct seqptr *sp, unsigned ntics)
 {
-	if (ntics > 0) {
-		sp->pos->delta += ntics;
-		sp->delta += ntics;
-		sp->tic += ntics;
-		statelist_outdate(&sp->statelist);
+	struct seqptr *link;
+
+	if (ntics == 0)
+		return;
+
+	sp->pos->delta += ntics;
+	sp->delta += ntics;
+	sp->tic += ntics;
+	statelist_outdate(&sp->statelist);
+
+	/* shift writer if affected */
+	link = sp->link;
+	if (link != NULL && link->pos == sp->pos && link->delta >= sp->delta) {
+		sp->link->delta += ntics;
+		sp->link->tic += ntics;
 	}
 }
 
@@ -348,6 +390,7 @@ unsigned
 seqptr_skip(struct seqptr *sp, unsigned ntics)
 {
 	unsigned delta;
+
 	while (ntics > 0) {
 		while (seqptr_evget(sp))
 			; /* nothing */
@@ -375,6 +418,164 @@ seqptr_seek(struct seqptr *sp, unsigned ntics)
 }
 
 
+/*
+ * move the next frame of the current tick to the given track. Must
+ * not be called on a non-frame-starting event.
+ */
+void
+seqptr_framerm(struct seqptr *sp, struct track *f)
+{
+	struct seqev **save_pos, *se, *spos, *fpos;
+	unsigned save_delta, sdelta, phase;
+
+	if (sp->delta != sp->pos->delta) {
+		log_puts("seqptr_framerm: not called at event position\n");
+		panic();
+	}
+	if (sp->pos->ev.cmd == EV_NULL) {
+		log_puts("seqptr_framerm: no event to remove\n");
+		panic();
+	}
+
+	track_clear(f);
+	fpos = f->first;
+
+	/*
+	 * Save current postition.
+	 */
+	save_pos = sp->pos->prev;
+	save_delta = sp->delta;
+
+	/*
+	 * Move all events that belong to the frame.
+	 */
+
+	spos = sp->pos;
+	sdelta = sp->delta;
+
+	phase = ev_phase(&spos->ev);
+
+	/* move event to frame track */
+	se = spos;
+	spos = se->next;
+	seqev_rm(se);
+	seqev_ins(fpos, se);
+
+	for (;;) {
+		if (phase & EV_PHASE_LAST)
+			break;
+		if (spos->ev.cmd == EV_NULL) {
+			/*
+			 * XXX: this can't happen, call panic() here ?
+			 */
+			log_puts("seqptr_framerm: unterminated frame\n");
+			track_clear(f);
+			return;
+		}
+
+		/* move to next event */
+		fpos->delta += spos->delta - sdelta;
+		sdelta = spos->delta;
+
+		/* process next event */
+		if (ev_match(&f->first->ev, &spos->ev)) {
+			phase = ev_phase(&spos->ev);
+
+			/* move event to frame track */
+			se = spos;
+			spos = se->next;
+			seqev_rm(se);
+			seqev_ins(fpos, se);
+		} else {
+			/* skip event */
+			spos = spos->next;
+			sdelta = 0;
+		}
+	}
+
+	/*
+	 * restore position.
+	 */
+	sp->pos = *save_pos;
+	sp->delta = save_delta;
+}
+
+/*
+ * insert the given frame at the end of the current tick. As the first
+ * event of the frame is not seen yet, the state list is not
+ * affected. As this is a stateless operation, it's the caller
+ * responsability to avoid conflicts.
+ */
+void
+seqptr_frameadd(struct seqptr *sp, struct track *f)
+{
+	struct seqev *se, *spos, **save_pos;
+	unsigned ntics, offs, sdelta, save_delta;
+
+	/*
+	 * Save current postition.
+	 */
+	save_pos = sp->pos->prev;
+	save_delta = sp->delta;
+
+	spos = sp->pos;
+	sdelta = sp->delta;
+	for (;;) {
+		if (f->first->ev.cmd == EV_NULL)
+			break;
+		se = f->first;
+		offs = se->delta;
+		se->delta = 0;
+		seqev_rm(se);
+
+		/*
+		 * move forward offs ticks, possibly inserting
+		 * empty space
+		 */
+		while (1) {
+			ntics = spos->delta - sdelta;
+			if (ntics > offs)
+				ntics = offs;
+			sdelta += ntics;
+			offs -= ntics;
+			if (offs == 0)
+				break;
+
+			/* if reached the end, append space */
+			if (spos->ev.cmd == EV_NULL) {
+				spos->delta += offs;
+				sdelta += offs;
+				offs = 0;
+				break;
+			}
+
+			spos = spos->next;
+			sdelta = 0;
+		}
+
+		/* make the event the last of the tick */
+		while (sdelta == spos->delta && spos->ev.cmd != EV_NULL) {
+			sdelta = 0;
+			spos = spos->next;
+		}
+
+		se->delta = sdelta;
+		spos->delta -= sdelta;
+		sdelta = 0;
+		/* link to the list */
+		se->next = spos;
+		se->prev = spos->prev;
+		*se->prev = se;
+		spos->prev = &se->next;
+	}
+
+	/*
+	 * Restore current position.
+	 */
+	sp->pos = *save_pos;
+	sp->delta = save_delta;
+}
+
 /*
  * generate an event that will suspend the frame of the given state;
  * the state is unchanged and may belong to any statelist.  Return 1
@@ -419,15 +620,16 @@ seqptr_restore(struct seqptr *sp, struct state *st)
  * the event never existed on the track. Returns the new state, or
  * NULL if there is no more state.
  */
-struct state *
-seqptr_rmlast(struct seqptr *sp, struct state *st)
+void
+seqptr_rmlast(struct seqptr *sp, struct state **pst)
 {
+	struct state *st = *pst;
 	struct seqev *i, *prev, *cur, *next;
 
 #ifdef FRAME_DEBUG
-	dbg_puts("seqptr_rmlast: ");
-	ev_dbg(&st->ev);
-	dbg_puts(" removing last event\n");
+	log_puts("seqptr_rmlast: ");
+	ev_log(&st->ev);
+	log_puts(" removing last event\n");
 #endif
 	/*
 	 * start a the first event of the frame and iterate until the
@@ -459,6 +661,7 @@ seqptr_rmlast(struct seqptr *sp, struct state *st)
 	next->prev = cur->prev;
 	*(cur->prev) = next;
 	seqev_del(cur);
+
 	/*
 	 * update the state; if we deleted the first event of the
 	 * frame, the state no more exists, so purge it
@@ -466,12 +669,11 @@ seqptr_rmlast(struct seqptr *sp, struct state *st)
 	if (prev == NULL) {
 		statelist_rm(&sp->statelist, st);
 		state_del(st);
-		return NULL;
+		*pst = NULL;
 	} else {
 		st->ev = prev->ev;
 		st->phase = st->pos == prev ? EV_PHASE_FIRST : EV_PHASE_NEXT;
 	}
-	return st;
 }
 
 /*
@@ -479,15 +681,16 @@ seqptr_rmlast(struct seqptr *sp, struct state *st)
  * position. Everything happens as the frame never existed on the
  * track. Returns always NULL, for consistency with seqptr_rmlast().
  */
-struct state *
-seqptr_rmprev(struct seqptr *sp, struct state *st)
+void
+seqptr_rmprev(struct seqptr *sp, struct state **pst)
 {
+	struct state *st = *pst;
 	struct seqev *i, *next;
 
 #ifdef FRAME_DEBUG
-	dbg_puts("seqptr_rmprev: ");
-	ev_dbg(&st->ev);
-	dbg_puts(" removing whole frame\n");
+	log_puts("seqptr_rmprev: ");
+	ev_log(&st->ev);
+	log_puts(" removing whole frame\n");
 #endif
 	/*
 	 * start a the first event of the frame and iterate until the
@@ -518,20 +721,17 @@ seqptr_rmprev(struct seqptr *sp, struct state *st)
 	}
 	statelist_rm(&sp->statelist, st);
 	state_del(st);
-	return NULL;
+	*pst = NULL;
 }
 
-
-
 /*
- * merge "low priority" event: check that the event of state "s1"
- * doen't conflict with event in thate "s2". If so, then it is put in
- * the track. Else "s1" is tagged as silent so a next call to this
- * routine will just skip it. This routine must be called before
- * seqptr_evmerge2() within the same tic.
+ * merge low priority event: if s1 doen't conflict with high priority
+ * events in the track, then store it in the track. Else drop it. This
+ * routine must be called before seqptr_evmerge2() within the current
+ * tick.
  */
-void
-seqptr_evmerge1(struct seqptr *pd, struct state *s1, struct state *s2)
+struct state *
+seqptr_evmerge1(struct seqptr *pd, struct state *s1)
 {
 	struct state *sd;
 
@@ -539,89 +739,86 @@ seqptr_evmerge1(struct seqptr *pd, struct state *s1, struct state *s2)
 	 * ignore bogus events
 	 */
 	if (s1->flags & (STATE_BOGUS | STATE_NESTED))
-		return;
-	if (s2 != NULL && s2->flags & (STATE_BOGUS | STATE_NESTED))
-		s2 = NULL;
+		return NULL;
 
-	if (s1->phase & EV_PHASE_FIRST) {
-		s1->tag = (!s2 || (s2->phase & EV_PHASE_LAST)) ? 1 : 0;
-#ifdef FRAME_DEBUG
-		if (!s1->tag) {
-			dbg_puts("seqptr_evmerge1: ");
-			ev_dbg(&s1->ev);
-			dbg_puts(" started in silent state\n");
-		}
-#endif
-	}
-	if (s1->tag) {
-		sd = statelist_lookup(&pd->statelist, &s1->ev);
-		if (!sd || !state_eq(sd, &s1->ev))
-			(void)seqptr_evput(pd, &s1->ev);
-	}
+	sd = statelist_lookup(&pd->statelist, &s1->ev);
+
+	if (sd != NULL) {
+		if (sd->tag == 0 && !(sd->phase & EV_PHASE_LAST))
+			return NULL;
+	} else if (!(s1->phase & EV_PHASE_FIRST))
+		return NULL;
+
+	if (sd == NULL || !state_eq(sd, &s1->ev))
+		sd = seqptr_evput(pd, &s1->ev);
+	sd->tag = 1;
+
+	return sd;
 }
 
 /*
- * merge "high priority" event: check that the event of state "s2"
- * doesn't conflict with events of state "s1". If so, then put it on
- * the track. If there is conflict then discard events related to "s1"
- * and put "s2". This routine must not be called before
- * seqptr_evmerge1() within the same tick.
+ * merge high priority event: if ev2 conflicts with low priority events
+ * in the track, discard them and store ev2. This routine must be
+ * called once seqptr_evmerge1() was called for all low prio events
+ * within the current tick.
  */
-void
-seqptr_evmerge2(struct seqptr *pd, struct state *s1, struct state *s2)
+unsigned
+seqptr_evmerge2(struct seqptr *pd, struct statelist *slist1, struct ev *ev2,
+	struct ev *ca)
 {
-	struct state *sd;
+	struct state *sd, *s1;
+	unsigned phase2;
+	unsigned rc = 0;
 
-	/*
-	 * ignore bogus events
-	 */
-	if (s2->flags & (STATE_BOGUS | STATE_NESTED))
-		return;
-	if (s1 != NULL && s1->flags & (STATE_BOGUS | STATE_NESTED))
-		s1 = NULL;
+	phase2 = ev_phase(ev2);
 
-	/*
-	 * tag/untag frames depending of if there are conflicts
-	 */
-	sd = statelist_lookup(&pd->statelist, &s2->ev);
-	if (s2->phase & EV_PHASE_FIRST) {
-		if (s1 && s1->tag) {
-			if (sd == NULL) {
-				dbg_puts("seqptr_merge2: ");
-				ev_dbg(&s1->ev);
-				dbg_puts(": no conflict\n");
-				dbg_panic();
+	sd = statelist_lookup(&pd->statelist, ev2);
+	if (sd != NULL) {
+		if (sd->tag == 0) {
+			/* EV_PHASE_LAST is never ambigous */
+			if (phase2 == EV_PHASE_LAST && !EV_ISNOTE(&sd->ev)) {
+				s1 = statelist_lookup(slist1, ev2);
+				if (s1 != NULL && s1->phase != EV_PHASE_LAST) {
+					if (!state_eq(s1, ev2))
+						sd = seqptr_evput(pd, &s1->ev);
+					sd->tag = 1;
+					return 0;
+				}
 			}
-			if (EV_ISNOTE(&s2->ev)) {
-				if (!(s1->phase & EV_PHASE_LAST))
-					sd = seqptr_rmprev(pd, sd);
-			} else {
-				if (s1->flags & STATE_CHANGED)
-					sd = seqptr_rmlast(pd, sd);
+		} else {
+			/*
+			 * phase could be (FIRST | NEXT) instead of
+			 * NEXT only, but in this case we wouldn't be
+			 * here because sd would be NULL.
+			 */
+			if (phase2 & EV_PHASE_FIRST) {
+				if (EV_ISNOTE(&sd->ev)) {
+					if (sd->phase != EV_PHASE_LAST) {
+						rc = state_cancel(sd, ca);
+						seqptr_rmprev(pd, &sd);
+					}
+				} else if (sd->flags & STATE_CHANGED)
+					seqptr_rmlast(pd, &sd);
 			}
-			s1->tag = 0;
 		}
-		s2->tag = 1;
-	} else if (s2->phase & EV_PHASE_NEXT) {
+	} else {
 		/*
-		 * nothing to do, conflicts already handled
+		 * If there's no state, this is necessarily the a new
+		 * frame
 		 */
-	} else if (s2->phase & EV_PHASE_LAST) {
-		if (s1 && !EV_ISNOTE(&s1->ev)) {
-			s2->tag = 0;
-			if (sd == NULL || !state_eq(sd, &s1->ev)) {
-				sd = seqptr_evput(pd, &s1->ev);
-			}
-			s1->tag = 1;
+		if (!(phase2 & EV_PHASE_FIRST)) {
+			log_puts("seqptr_evmerge2: ");
+			ev_log(ev2);
+			log_puts(": missing state\n");
+			panic();
 		}
 	}
 
-	/*
-	 * store the event, if the frame is tagged
-	 */
-	if (s2->tag && (sd == NULL || !state_eq(sd, &s2->ev))) {
-		(void)seqptr_evput(pd, &s2->ev);
-	}
+	if (sd == NULL || !state_eq(sd, ev2))
+		sd = seqptr_evput(pd, ev2);
+	sd->tag = 0;
+
+	return rc;
 }
 
 /*
@@ -631,10 +828,11 @@ seqptr_evmerge2(struct seqptr *pd, struct state *s1, struct state *s2)
 void
 track_merge(struct track *dst, struct track *src)
 {
-	struct state *s1, *s2;
+	struct state *s;
 	struct seqptr *p2, *pd;
 	struct statelist orglist;
 	unsigned delta1, delta2, deltad;
+	struct ev ca;
 
 	pd = seqptr_new(dst);
 	p2 = seqptr_new(src);
@@ -649,11 +847,10 @@ track_merge(struct track *dst, struct track *src)
 		 * 'dst' track.
 		 */
 		for(;;) {
-			s1 = seqptr_evdel(pd, &orglist);
-			if (s1 == NULL)
+			s = seqptr_evdel(pd, &orglist);
+			if (s == NULL)
 				break;
-			s2 = statelist_lookup(&p2->statelist, &s1->ev);
-			seqptr_evmerge1(pd, s1, s2);
+			seqptr_evmerge1(pd, s);
 		}
 
 		/*
@@ -661,11 +858,10 @@ track_merge(struct track *dst, struct track *src)
 		 * with the original state of 'dst'.
 		 */
 		for (;;) {
-			s2 = seqptr_evget(p2);
-			if (s2 == NULL)
+			s = seqptr_evget(p2);
+			if (s == NULL)
 				break;
-			s1 = statelist_lookup(&orglist, &s2->ev);
-			seqptr_evmerge2(pd, s1, s2);
+			seqptr_evmerge2(pd, &orglist, &s->ev, &ca);
 		}
 
 		/*
@@ -741,7 +937,7 @@ track_move(struct track *src, unsigned start, unsigned len,
 	 */
 	if (blank) {
 		for (st = slist.first; st != NULL; st = st->next) {
-			if (state_inspec(st, es) &&
+			if (!EV_ISNOTE(&st->ev) && state_inspec(st, es) &&
 			    seqptr_cancel(sp, st))
 				st->tag &= ~TAG_KEEP;
 		}
@@ -779,7 +975,7 @@ track_move(struct track *src, unsigned start, unsigned len,
 	 */
 	if (copy) {
 		for (st = slist.first; st != NULL; st = st->next) {
-			if (!state_inspec(st, es))
+			if (EV_ISNOTE(&st->ev) || !state_inspec(st, es))
 				continue;
 			if (!(st->tag & TAG_COPY) && seqptr_restore(dp, st)) {
 				st->tag |= TAG_COPY;
@@ -898,7 +1094,8 @@ track_move(struct track *src, unsigned start, unsigned len,
  * quantize the given track
  */
 void
-track_quantize(struct track *src, unsigned start, unsigned len,
+track_quantize(struct track *src, struct evspec *es,
+    unsigned start, unsigned len,
     unsigned offset, unsigned quant, unsigned rate) {
 	unsigned tic, qtic;
 	struct track qt;
@@ -952,21 +1149,22 @@ track_quantize(struct track *src, unsigned start, unsigned len,
 		delta = tic + ofs - qtic;
 #ifdef FRAME_DEBUG
 		if (delta < 0) {
-			dbg_puts("track_quantize: delta < 0\n");
-			dbg_panic();
+			log_puts("track_quantize: delta < 0\n");
+			panic();
 		}
 #endif
 		seqptr_ticput(qp, delta);
 		qtic += delta;
 
 		if (st->phase & EV_PHASE_FIRST) {
-			if (EV_ISNOTE(&st->ev)) {
+			if (evspec_matchev(es, &st->ev)) {
 				st->tag = 1;
-				fluct += (ofs < 0) ? -ofs : ofs;
-				notes++;
-			} else {
+				if (EV_ISNOTE(&st->ev)) {
+					fluct += (ofs < 0) ? -ofs : ofs;
+					notes++;
+				}
+			} else
 				st->tag = 0;
-			}
 		}
 		if (st->tag) {
 			seqptr_evput(qp, &st->ev);
@@ -999,36 +1197,173 @@ track_quantize(struct track *src, unsigned start, unsigned len,
 	seqptr_del(qp);
 	track_done(&qt);
 	if (notes > 0) {
-		dbg_puts("track_quantize: fluct = ");
-		dbg_putu(fluct);
-		dbg_puts(", notes = ");
-		dbg_putu(notes);
-		dbg_puts(", avg = ");
-		dbg_putu(100 * fluct / notes);
-		dbg_puts("% of a tick\n");
+		log_puts("quantize: ");
+		log_putu(notes);
+		log_puts(" notes, fluctuation = ");
+		log_putu(100 * fluct / notes);
+		log_puts("% of a tick\n");
 	}
 }
 
 /*
- * time-scale the given track in such a way that
- * 'oldu' tics will correspond to 'newu'
+ * quantize the given track by moving frames
  */
 void
-track_scale(struct track *t, unsigned oldunit, unsigned newunit)
+track_quantize_frame(struct track *src, struct evspec *es,
+    unsigned start, unsigned len,
+    unsigned offset, unsigned quant, unsigned rate) {
+	unsigned tic, qtic;
+	struct track qt, frame;
+	struct seqptr *sp, *qp;
+	struct state *st;
+	unsigned remaind;
+	unsigned fluct, notes;
+	int ofs, delta;
+
+	sp = seqptr_new(src);
+
+	/*
+	 * go to start position
+	 */
+	if (seqptr_skip(sp, start) > 0) {
+		seqptr_del(sp);
+		return;
+	}
+
+	track_init(&qt);
+	qp = seqptr_new(&qt);
+	seqptr_seek(qp, start);
+
+	track_init(&frame);
+	tic = qtic = start;
+	ofs = 0;
+
+	/*
+	 * go ahead and copy all events to quantize during 'len' tics,
+	 * while stretching the time scale in the destination track
+	 */
+	fluct = 0;
+	notes = 0;
+	for (;;) {
+		delta = seqptr_ticskip(sp, ~0U);
+		tic += delta;
+		if (tic >= start + len)
+			break;
+
+		remaind = quant != 0 ? (tic - start + offset) % quant : 0;
+		if (remaind < quant / 2) {
+			ofs = - ((remaind * rate + 99) / 100);
+		} else {
+			ofs = ((quant - remaind) * rate + 99) / 100;
+		}
+
+		delta = tic + ofs - qtic;
+#ifdef FRAME_DEBUG
+		if (delta < 0) {
+			log_puts("track_quantize_frame: delta < 0\n");
+			panic();
+		}
+#endif
+		seqptr_seek(qp, delta);
+		qtic += delta;
+
+		if (seqptr_eot(sp))
+			break;
+
+		st = statelist_lookup(&sp->statelist, &sp->pos->ev);
+		if (st != NULL && !(st->phase & EV_PHASE_LAST)) {
+			/*
+			 * There's a state for this event in the
+			 * source track, which means the frame
+			 * is not being quantized
+			 */
+#ifdef FRAME_DEBUG
+			if (sp->pos->ev.cmd != EV_NULL) {
+				ev_log(&sp->pos->ev);
+				log_puts(": skipped (not ours)\n");
+			}
+#endif
+			seqptr_evget(sp);
+			continue;
+		}
+
+		st = statelist_lookup(&qp->statelist, &sp->pos->ev);
+		if (st != NULL && !(st->phase & EV_PHASE_LAST)) {
+			/*
+			 * There's as state for this event, which is
+			 * part of a conflicting frame. Just skip it.
+			 */
+#ifdef FRAME_DEBUG
+			if (sp->pos->ev.cmd != EV_NULL) {
+				ev_log(&sp->pos->ev);
+				log_puts(": skipped (conflict)\n");
+			}
+#endif
+			seqptr_evget(sp);
+			continue;
+		}
+
+		if (!evspec_matchev(es, &sp->pos->ev)) {
+			/*
+			 * Doesn't match selection, Skip this event.
+			 */
+			seqptr_evget(sp);
+			continue;
+		}
+
+		seqptr_framerm(sp, &frame);
+		if (EV_ISNOTE(&frame.first->ev)) {
+			fluct += (ofs < 0) ? -ofs : ofs;
+			notes++;
+		}
+		seqptr_frameadd(qp, &frame);
+		while (seqptr_evget(qp))
+			;
+	}
+
+	statelist_empty(&sp->statelist);
+	statelist_empty(&qp->statelist);
+	seqptr_del(sp);
+	seqptr_del(qp);
+	track_done(&frame);
+
+	track_merge(src, &qt);
+	track_done(&qt);
+
+	if (notes > 0) {
+		log_puts("quantize: ");
+		log_putu(notes);
+		log_puts(" notes, fluctuation = ");
+		log_putu(100 * fluct / notes);
+		log_puts("% of a tick\n");
+	}
+}
+
+/*
+ * First step to time-scale of the given track: round event positions
+ * and convert tempo/signature events without scaling the track.
+ */
+void
+track_prescale(struct track *t, unsigned oldunit, unsigned newunit)
 {
 	struct seqptr *sp;
 	struct statelist slist;
 	struct state *st;
 	struct ev ev;
-	unsigned delta, err;
+	unsigned delta, round, err;
+
+	round = oldunit / newunit;
+	if (round == 0)
+		round = 1;
 
 	err = 0;
 	sp = seqptr_new(t);
 	statelist_init(&slist);
 	for (;;) {
-		delta = newunit * seqptr_ticdel(sp, ~0U, &slist) + err;
-		seqptr_ticput(sp, delta / oldunit);
-		err = delta % oldunit;
+		delta = seqptr_ticdel(sp, ~0U, &slist) + err;
+		err = delta % round;
+		delta -= err;
+		seqptr_ticput(sp, delta);
 		st = seqptr_evdel(sp, &slist);
 		if (st == NULL) {
 			break;
@@ -1054,11 +1389,37 @@ track_scale(struct track *t, unsigned oldunit, unsigned newunit)
 	seqptr_del(sp);
 }
 
+/*
+ * Finalize time-scaling the given track in such a way that 'oldunit'
+ * ticks will correspond to 'newunit'.
+ */
+void
+track_scale(struct track *t, unsigned oldunit, unsigned newunit)
+{
+	struct seqptr *sp;
+	struct statelist slist;
+	struct state *st;
+	unsigned delta;
+
+	sp = seqptr_new(t);
+	statelist_init(&slist);
+	for (;;) {
+		delta = newunit * seqptr_ticdel(sp, ~0U, &slist);
+		seqptr_ticput(sp, delta / oldunit);
+		st = seqptr_evdel(sp, &slist);
+		if (st == NULL)
+			break;
+		seqptr_evput(sp, &st->ev);
+	}
+	statelist_done(&slist);
+	seqptr_del(sp);
+}
+
 /*
  * transpose the given track
  */
 void
-track_transpose(struct track *src, unsigned start, unsigned len, 
+track_transpose(struct track *src, unsigned start, unsigned len,
     struct evspec *es, int halftones)
 {
 	unsigned delta, tic;
@@ -1141,6 +1502,95 @@ track_transpose(struct track *src, unsigned start, unsigned len,
 	track_done(&qt);
 }
 
+/*
+ * apply velocity curve to given track
+ */
+void
+track_vcurve(struct track *src, unsigned start, unsigned len,
+    struct evspec *es, int weight)
+{
+	unsigned delta, tic;
+	struct seqptr *sp;
+	struct state *st;
+	struct statelist slist;
+	struct ev ev;
+
+	/* put weight from -63:63 to 1:127 range */
+	weight = (64 - weight) & 0x7f;
+
+	sp = seqptr_new(src);
+	statelist_dup(&slist, &sp->statelist);
+	tic = 0;
+
+	/*
+	 * rewrite all events, modifying selected ones
+	 */
+	for (;;) {
+		delta = seqptr_ticdel(sp, ~0U, &slist);
+		seqptr_ticput(sp, delta);
+		st = seqptr_evdel(sp, &slist);
+		if (st == NULL)
+			break;
+		tic += delta;
+		if ((st->phase & EV_PHASE_FIRST) &&
+		    tic >= start && tic < start + len &&
+		    EV_ISNOTE(&st->ev) && state_inspec(st, es)) {
+			ev = st->ev;
+			ev.note_vel = vcurve(weight, ev.note_vel);
+			seqptr_evput(sp, &ev);
+		} else {
+			seqptr_evput(sp, &st->ev);
+		}
+	}
+
+	statelist_done(&slist);
+	seqptr_del(sp);
+}
+
+/*
+ * rewrite the track frame-by-frame
+ */
+void
+track_rewrite(struct track *src)
+{
+	struct statelist slist;
+	struct track dst, frame;
+	struct seqptr *sp, *dp;
+	unsigned delta;
+
+	track_check(src);
+
+	track_init(&frame);
+
+	sp = seqptr_new(src);
+	statelist_init(&slist);
+
+	track_init(&dst);
+	dp = seqptr_new(&dst);
+
+	/*
+	 * reconstruct frame-by-frame
+	 */
+	for (;;) {
+		delta = seqptr_ticdel(sp, ~0U, &slist);
+		seqptr_seek(dp, delta);
+
+		if (seqptr_eot(sp))
+			break;
+
+		seqptr_framerm(sp, &frame);
+		seqptr_frameadd(dp, &frame);
+	}
+
+	statelist_empty(&dp->statelist);
+	seqptr_del(dp);
+	statelist_done(&slist);
+	seqptr_del(sp);
+
+	track_swap(src, &dst);
+	track_done(&dst);
+}
+
 /*
  * check (and fix) the given track for inconsistencies
  */
@@ -1169,14 +1619,14 @@ track_check(struct track *src)
 		}
 		if (st->phase & EV_PHASE_FIRST) {
 			if (st->flags & STATE_BOGUS) {
-				dbg_puts("track_check: ");
-				ev_dbg(&st->ev);
-				dbg_puts(": bogus\n");
+				log_puts("track_check: ");
+				ev_log(&st->ev);
+				log_puts(": bogus\n");
 				st->tag = 0;
 			} else if (st->flags & STATE_NESTED) {
-				dbg_puts("track_check: ");
-				ev_dbg(&st->ev);
-				dbg_puts(": nested\n");
+				log_puts("track_check: ");
+				ev_log(&st->ev);
+				log_puts(": nested\n");
 				st->tag = 0;
 			} else {
 				st->tag = 1;
@@ -1190,9 +1640,9 @@ track_check(struct track *src)
 			if (dst == NULL || !state_eq(dst, &st->ev)) {
 				seqptr_evput(sp, &st->ev);
 			} else {
-				dbg_puts("track_check: ");
-				ev_dbg(&st->ev);
-				dbg_puts(": duplicated\n");
+				log_puts("track_check: ");
+				ev_log(&st->ev);
+				log_puts(": duplicated\n");
 			}
 		}
 	}
@@ -1203,10 +1653,10 @@ track_check(struct track *src)
 	for (st = sp->statelist.first; st != NULL; st = stnext) {
 		stnext = st->next;
 		if (!(st->phase & EV_PHASE_LAST)) {
-			dbg_puts("track_check: ");
-			ev_dbg(&st->ev);
-			dbg_puts(": unterminated\n");
-			(void)seqptr_rmprev(sp, st);
+			log_puts("track_check: ");
+			ev_log(&st->ev);
+			log_puts(": unterminated\n");
+			seqptr_rmprev(sp, &st);
 		}
 	}
 
@@ -1294,11 +1744,11 @@ track_findmeasure(struct track *t, unsigned m)
 	seqptr_del(sp);
 
 #ifdef FRAME_DEBUG
-	dbg_puts("track_findmeasure: ");
-	dbg_putu(m);
-	dbg_puts(" -> ");
-	dbg_putu(tic);
-	dbg_puts("\n");
+	log_puts("track_findmeasure: ");
+	log_putu(m);
+	log_puts(" -> ");
+	log_putu(tic);
+	log_puts("\n");
 #endif
 
 	return tic;
@@ -1412,10 +1862,10 @@ track_confev(struct track *src, struct ev *ev)
 	struct state *st;
 
 	if (ev_phase(ev) != (EV_PHASE_FIRST | EV_PHASE_LAST)) {
-		dbg_puts("track_confev: ");
-		ev_dbg(ev);
-		dbg_puts(": bad phase, ignored");
-		dbg_puts("\n");
+		log_puts("track_confev: ");
+		ev_log(ev);
+		log_puts(": bad phase, ignored");
+		log_puts("\n");
 		return;
 	}
 
@@ -1524,7 +1974,7 @@ track_cut(struct track *t, unsigned stic, unsigned len)
  * map current selection to the given evspec
  */
 void
-track_evmap(struct track *src, unsigned start, unsigned len, 
+track_evmap(struct track *src, unsigned start, unsigned len,
     struct evspec *es, struct evspec *from, struct evspec *to)
 {
 	unsigned delta, tic;
