@@ -59,7 +59,7 @@
 #define MIDI_BUFSIZE	1024
 #define MAXFDS		(DEFAULT_MAXNDEVS + 1)
 
-volatile sig_atomic_t cons_quit = 0, resize_flag = 0, cont_flag = 0;
+volatile sig_atomic_t cons_quit = 0, resize_flag = 0, cont_flag = 0, usr1_flag = 0;
 struct timespec ts, ts_last;
 
 int cons_eof, cons_isatty;
@@ -103,6 +103,12 @@ mdep_sigcont(int s)
 	cont_flag = 1;
 }
 
+void
+mdep_sigusr1(int s)
+{
+	usr1_flag = 1;
+}
+
 /*
  * start the mux, must be called just after devices are opened
  */
@@ -185,15 +191,6 @@ mux_mdep_wait(int docons)
 		}
 	} else
 		tty_pfds = NULL;
-	for (dev = mididev_list; dev != NULL; dev = dev->next) {
-		if (!(dev->mode & MIDIDEV_MODE_IN) || dev->eof) {
-			dev->pfd = NULL;
-			continue;
-		}
-		pfd = &pfds[nfds];
-		nfds += dev->ops->pollfd(dev, pfd, POLLIN);
-		dev->pfd = pfd;
-	}
 	if (cons_quit) {
 		fprintf(stderr, "\n--interrupt--\n");
 		cons_quit = 0;
@@ -211,6 +208,28 @@ mux_mdep_wait(int docons)
 		if (cons_isatty)
 			tty_reset();
 	}
+	if (usr1_flag) {
+		usr1_flag = 0;
+		for (dev = mididev_list; dev != NULL; dev = dev->next) {
+			if (dev->eof) {
+				mididev_close(dev);
+				mididev_open(dev);
+				if (!dev->eof) {
+					log_puti(dev->unit);
+					log_puts(": device reopened\n");
+				}
+			}
+		}
+	}
+	for (dev = mididev_list; dev != NULL; dev = dev->next) {
+		if (!(dev->mode & MIDIDEV_MODE_IN) || dev->eof) {
+			dev->pfd = NULL;
+			continue;
+		}
+		pfd = &pfds[nfds];
+		nfds += dev->ops->pollfd(dev, pfd, POLLIN);
+		dev->pfd = pfd;
+	}
 	res = poll(pfds, nfds, -1);
 	if (res < 0 && errno != EINTR) {
 		log_perror("mux_mdep_wait: poll");
@@ -370,6 +389,11 @@ cons_init(struct el_ops *el_ops, void *el_arg)
 		log_perror("cons_mdep_init: sigaction(cont) failed");
 		exit(1);
 	}
+	sa.sa_handler = mdep_sigusr1;
+	if (sigaction(SIGUSR1, &sa, NULL) < 0) {
+		log_perror("cons_mdep_init: sigaction(usr1) failed");
+		exit(1);
+	}
 	if (!user_flag_batch && !user_flag_verb && tty_init()) {
 		cons_isatty = 1;
 		el_init(el_ops, el_arg);
@@ -403,6 +427,10 @@ cons_done(void)
 		log_perror("cons_mdep_done: sigaction(cont)");
 		exit(1);
 	}
+	if (sigaction(SIGUSR1, &sa, NULL) < 0) {
+		log_perror("cons_mdep_done: sigaction(usr1)");
+		exit(1);
+	}
 }
 
 /*
