@@ -1,34 +1,20 @@
 /*
  * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "mux.h"
 #include "metro.h"
 #include "song.h"
@@ -76,8 +62,8 @@ metro_tocb(void *addr)
 	struct ev ev;
 
 	if (o->ev == NULL) {
-		dbg_puts("metro_tocb: no click sounding\n");
-		dbg_panic();
+		log_puts("metro_tocb: no click sounding\n");
+		panic();
 	}
 	ev.cmd = EV_NOFF;
 	ev.dev = o->ev->dev;
@@ -102,7 +88,7 @@ metro_tic(struct metro *o, unsigned beat, unsigned tic)
 		 * abord the timeout and stop the click
 		 */
 		if (o->ev) {
-			dbg_puts("metro_tic: nested clicks\n");
+			log_puts("metro_tic: nested clicks\n");
 			timo_del(&o->to);
 			metro_tocb(o);
 		}
