@@ -1,31 +1,17 @@
 /*
  * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -33,19 +19,21 @@
  * node of the tree represents one instruction.
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "str.h"
 #include "data.h"
 #include "node.h"
 #include "exec.h"
 #include "cons.h"
+#include "user.h"
+#include "textio.h"
 
 struct node *
 node_new(struct node_vmt *vmt, struct data *data)
 {
 	struct node *o;
 
-	o = mem_alloc(sizeof(struct node), "node");
+	o = xmalloc(sizeof(struct node), "node");
 	o->vmt = vmt;
 	o->data = data;
 	o->list = o->next = NULL;
@@ -66,35 +54,35 @@ node_delete(struct node *o)
 	if (o->data) {
 		data_delete(o->data);
 	}
-	mem_free(o);
+	xfree(o);
 }
 
 void
-node_dbg(struct node *o, unsigned depth)
+node_log(struct node *o, unsigned depth)
 {
 #define NODE_MAXDEPTH 30
 	static char str[2 * NODE_MAXDEPTH + 1] = "";
 	struct node *i;
 
-	dbg_puts(str);
-	dbg_puts(o != NULL && o->next != NULL ? "+-" : "\\-");
+	log_puts(str);
+	log_puts(o != NULL && o->next != NULL ? "+-" : "\\-");
 	if (o == NULL) {
-		dbg_puts("<EMPTY>\n");
+		log_puts("<EMPTY>\n");
 		return;
 	}
-	dbg_puts(o->vmt->name);
+	log_puts(o->vmt->name);
 	if (o->data) {
-		dbg_puts("(");
-		data_dbg(o->data);
-		dbg_puts(")");
+		log_puts("(");
+		data_log(o->data);
+		log_puts(")");
 	}
-	dbg_puts(depth >= NODE_MAXDEPTH && o->list ? "[...]\n" : "\n");
+	log_puts(depth >= NODE_MAXDEPTH && o->list ? "[...]\n" : "\n");
 	if (depth < NODE_MAXDEPTH) {
 		str[2 * depth] = o->next ? '|' : ' ';
 		str[2 * depth + 1] = ' ';
 		str[2 * depth + 2] = '\0';
 		for (i = o->list; i != NULL; i = i->next) {
-			node_dbg(i, depth + 1);
+			node_log(i, depth + 1);
 		}
 		str[2 * depth] = '\0';
 	}
@@ -113,8 +101,8 @@ void
 node_replace(struct node **n, struct node *e)
 {
 	if (e->list != NULL) {
-		dbg_puts("node_replace: e->list != NULL\n");
-		dbg_panic();
+		log_puts("node_replace: e->list != NULL\n");
+		panic();
 	}
 	e->list = *n;
 	e->next = (*n)->next;
@@ -302,6 +290,10 @@ node_exec_ignore(struct node *o, struct exec *x, struct data **r)
 	if (result == RESULT_ERR || result == RESULT_EXIT) {
 		return result;
 	}
+	if (*r != NULL && (*r)->type != DATA_NIL) {
+		data_print(*r);
+		textout_putstr(tout, "\n");
+	}
 	data_delete(*r);
 	*r = NULL;
 	return RESULT_OK;
@@ -317,6 +309,7 @@ node_exec_call(struct node *o, struct exec *x, struct data **r)
 	struct name **oldlocals, *newlocals;
 	struct name *argn;
 	struct node *argv;
+	struct var *valist;
 	char *procname_save;
 	unsigned result;
 
@@ -328,8 +321,13 @@ node_exec_call(struct node *o, struct exec *x, struct data **r)
 		cons_errs(o->data->val.ref, "no such proc");
 		goto finish;
 	}
+	valist = NULL;
 	argv = o->list;
 	for (argn = p->args; argn != NULL; argn = argn->next) {
+		if (str_eq(argn->str, "...")) {
+			valist = var_new(&newlocals, "...", data_newlist(NULL));
+			break;
+		}
 		if (argv == NULL) {
 			cons_errs(o->data->val.ref, "to few arguments");
 			goto finish;
@@ -341,10 +339,17 @@ node_exec_call(struct node *o, struct exec *x, struct data **r)
 		argv = argv->next;
 		*r = NULL;
 	}
-	if (argv != NULL) {
+	if (valist == NULL && argv != NULL) {
 		cons_errs(o->data->val.ref, "to many arguments");
 		goto finish;
 	}
+	while (argv != NULL) {
+		if (node_exec(argv, x, r) == RESULT_ERR)
+			goto finish;
+		data_listadd(valist->data, *r);
+		argv = argv->next;
+		*r = NULL;
+	}
 	oldlocals = x->locals;
 	x->locals = &newlocals;
 	procname_save = x->procname;
@@ -395,11 +400,13 @@ node_exec_for(struct node *o, struct exec *x, struct data **r)
 	unsigned result;
 	struct data *list, *i;
 	struct var *v;
+
 	if (node_exec(o->list, x, &list) == RESULT_ERR) {
 		return RESULT_ERR;
 	}
-	if (list->type != DATA_LIST) {
-		cons_errs(x->procname, "argument to 'for' must be a list");
+	if (list->type != DATA_LIST && list->type != DATA_RANGE) {
+		cons_errs(x->procname,
+		    "argument to 'for' must be a list or range");
 		return RESULT_ERR;
 	}
 	v = exec_varlookup(x, o->data->val.ref);
@@ -407,15 +414,36 @@ node_exec_for(struct node *o, struct exec *x, struct data **r)
 		v = var_new(x->locals, o->data->val.ref, data_newnil());
 	}
 	result = RESULT_OK;
-	for (i = list->val.list; i != NULL; i = i->next) {
-		data_assign(v->data, i);
-		result = node_exec(o->list->next, x, r);
-		if (result == RESULT_CONTINUE) {
-			continue;
-		} else if (result == RESULT_BREAK) {
-			break;
-		} else if (result != RESULT_OK) {
-			break;
+	if (list->type == DATA_LIST) {
+		for (i = list->val.list; i != NULL; i = i->next) {
+			data_assign(v->data, i);
+			result = node_exec(o->list->next, x, r);
+			if (result == RESULT_CONTINUE) {
+				continue;
+			} else if (result == RESULT_BREAK) {
+				break;
+			} else if (result != RESULT_OK) {
+				break;
+			}
+		}
+	} else {
+		if (list->val.range.min <= list->val.range.max) {
+			i = data_newlong(list->val.range.min);
+			while (1) {
+				data_assign(v->data, i);
+				result = node_exec(o->list->next, x, r);
+				if (result == RESULT_CONTINUE) {
+					continue;
+				} else if (result == RESULT_BREAK) {
+					break;
+				} else if (result != RESULT_OK) {
+					break;
+				}
+				if (i->val.num == list->val.range.max)
+					break;
+				i->val.num++;
+			}
+			data_delete(i);
 		}
 	}
 	data_delete(list);
