@@ -1,31 +1,17 @@
 /*
- * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
+ * Copyright (c) 2003-2019 Alexandre Ratchov <alex@caoua.org>
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -45,7 +31,7 @@
  *
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "pool.h"
 #include "state.h"
 
@@ -81,29 +67,29 @@ state_del(struct state *s)
  * dump the state to stderr
  */
 void
-state_dbg(struct state *s)
+state_log(struct state *s)
 {
-	ev_dbg(&s->ev);
+	ev_log(&s->ev);
 	if (s->flags & STATE_NEW) {
-		dbg_puts(" NEW");
+		log_puts(" NEW");
 	}
 	if (s->flags & STATE_CHANGED) {
-		dbg_puts(" CHANGED");
+		log_puts(" CHANGED");
 	}
 	if (s->flags & STATE_BOGUS) {
-		dbg_puts(" BOGUS");
+		log_puts(" BOGUS");
 	}
 	if (s->flags & STATE_NESTED) {
-		dbg_puts(" NESTED");
+		log_puts(" NESTED");
 	}
 	if (s->phase & EV_PHASE_FIRST) {
-		dbg_puts(" FIRST");
+		log_puts(" FIRST");
 	}
 	if (s->phase & EV_PHASE_NEXT) {
-		dbg_puts(" NEXT");
+		log_puts(" NEXT");
 	}
 	if (s->phase & EV_PHASE_LAST) {
-		dbg_puts(" LAST");
+		log_puts(" LAST");
 	}
 }
 
@@ -126,55 +112,17 @@ state_copyev(struct state *st, struct ev *ev, unsigned ph)
 unsigned
 state_match(struct state *st, struct ev *ev)
 {
-	switch (st->ev.cmd) {
-	case EV_NON:
-	case EV_NOFF:
-	case EV_KAT:
-		if (!EV_ISNOTE(ev) ||
-		    st->ev.note_num != ev->note_num ||
-		    st->ev.ch != ev->ch ||
-		    st->ev.dev != ev->dev) {
-			return 0;
-		}
-		break;
-	case EV_XCTL:
-	case EV_NRPN:
-	case EV_RPN:
-		if (st->ev.cmd != ev->cmd ||
-		    st->ev.dev != ev->dev ||
-		    st->ev.ch != ev->ch ||
-		    st->ev.v0 != ev->v0) {
-			return 0;
-		}
-		break;
-	case EV_BEND:
-	case EV_CAT:
-	case EV_XPC:
-		if (st->ev.cmd != ev->cmd ||
-		    st->ev.dev != ev->dev ||
-		    st->ev.ch != ev->ch) {
-			return 0;
-		}
-		break;
-	case EV_TEMPO:
-	case EV_TIMESIG:
-		if (st->ev.cmd != ev->cmd) {
-			return 0;
-		}
-		break;
-	default:
-		dbg_puts("state_match: ");
-		state_dbg(st);
-		dbg_puts(": bad event type\n");
-		dbg_panic();
-		break;
-	}
+	unsigned res;
+
+	res = ev_match(&st->ev, ev);
 #ifdef STATE_DEBUG
-	dbg_puts("state_match: ");
-	ev_dbg(&st->ev);
-	dbg_puts(": ok\n");
+	if (res) {
+		log_puts("state_match: ");
+		ev_log(&st->ev);
+		log_puts(": ok\n");
+	}
 #endif
-	return 1;
+	return res;
 }
 
 /*
@@ -183,71 +131,45 @@ state_match(struct state *st, struct ev *ev)
 unsigned
 state_inspec(struct state *st, struct evspec *spec)
 {
+	struct evinfo *ei;
+
 	if (spec == NULL) {
 		return 1;
 	}
+	ei = evinfo + st->ev.cmd;
 	switch(spec->cmd) {
 	case EVSPEC_EMPTY:
 		return 0;
 	case EVSPEC_ANY:
 		goto ch;
 	case EVSPEC_NOTE:
-		if (!(st->phase & EV_PHASE_FIRST)) {
-			dbg_puts("state_inspec: not first\n");
-			dbg_panic();
-		}
-		if (st->ev.cmd == EV_NON) {
-			goto v0;
-		}
-		break;
-	case EVSPEC_CAT:
-		if (st->ev.cmd == EV_CAT) {
-			goto ch;
-		}
-		break;
-	case EVSPEC_BEND:
-		if (st->ev.cmd == EV_BEND) {
-			goto ch;
-		}
-		break;
-	case EVSPEC_XPC:
-		if (st->ev.cmd == EV_XPC) {
-			goto v1;
-		}
-		break;
-	case EVSPEC_NRPN:
-		if (st->ev.cmd == EV_NRPN) {
-			goto v1;
-		}
-		break;
-	case EVSPEC_RPN:
-		if (st->ev.cmd == EV_RPN) {
-			goto v1;
-		}
-		break;
-	case EVSPEC_XCTL:
-		if (st->ev.cmd == EV_XCTL) {
-			goto v0;
-		}
+		if (!EV_ISNOTE(&st->ev))
+			return 0;
 		break;
 	default:
-		break;
+		if (st->ev.cmd != spec->cmd)
+			return 0;
 	}
-	return 0;
-
-v1:	if (st->ev.v1 < spec->v1_min ||
-	    st->ev.v1 > spec->v1_max) {
-		return 0;
+	if (ei->nparams >= 1) {
+		if (st->ev.v0 < spec->v0_min ||
+		    st->ev.v0 > spec->v0_max)
+			return 0;
 	}
-v0:	if (st->ev.v0 < spec->v0_min ||
-	    st->ev.v0 > spec->v0_max) {
-		return 0;
+	if (ei->nparams >= 2) {
+		if (st->ev.v1 < spec->v1_min ||
+		    st->ev.v1 > spec->v1_max)
+			return 0;
 	}
-ch:	if (st->ev.dev < spec->dev_min ||
-	    st->ev.dev > spec->dev_max ||
-	    st->ev.ch < spec->ch_min ||
-	    st->ev.ch > spec->ch_max) {
-		return 0;
+ch:
+	if (ei->flags & EV_HAS_DEV) {
+		if (st->ev.dev < spec->dev_min ||
+		    st->ev.dev > spec->dev_max)
+			return 0;
+	}
+	if (ei->flags & EV_HAS_CH) {
+		if (st->ev.ch < spec->ch_min ||
+		    st->ev.ch > spec->ch_max)
+			return 0;
 	}
 	return 1;
 }
@@ -259,6 +181,8 @@ ch:	if (st->ev.dev < spec->dev_min ||
 unsigned
 state_eq(struct state *st, struct ev *ev)
 {
+	struct evinfo *ei;
+
 	if (EV_ISVOICE(&st->ev)) {
 		switch(st->ev.cmd) {
 		case EV_CAT:
@@ -273,6 +197,13 @@ state_eq(struct state *st, struct ev *ev)
 				return 0;
 			break;
 		}
+	} else if (EV_ISSX(&st->ev)) {
+		if (st->ev.cmd != ev->cmd)
+			return 0;
+		ei = evinfo + st->ev.cmd;
+		if ((ei->nparams >= 1 && st->ev.v0 != ev->v0) ||
+		    (ei->nparams >= 2 && st->ev.v1 != ev->v1))
+			return 0;
 	} else if (st->ev.cmd == EV_TEMPO) {
 		if (st->ev.tempo_usec24 != ev->tempo_usec24) {
 			return 0;
@@ -283,8 +214,8 @@ state_eq(struct state *st, struct ev *ev)
 			return 0;
 		}
 	} else {
-		dbg_puts("state_eq: not defined\n");
-		dbg_panic();
+		log_puts("state_eq: not defined\n");
+		panic();
 	}
 	return 1;
 }
@@ -337,8 +268,8 @@ state_cancel(struct state *st, struct ev *rev)
 		 * other events have their EV_PHASE_LAST bit set, so
 		 * we never come here
 		 */
-		dbg_puts("state_cancel: unknown event type\n");
-		dbg_panic();
+		log_puts("state_cancel: unknown event type\n");
+		panic();
 	}
 	return 1;
 }
@@ -362,15 +293,16 @@ state_restore(struct state *st, struct ev *rev)
 		 * we never use this function for NOTE events, so
 		 * if we're here, there is problem somewhere...
 		 */
-		dbg_puts("state_restore: can't restore note events\n");
-		dbg_panic();
+		log_puts("state_restore: can't restore note events\n");
+		panic();
 	}
-	if ((st->phase & EV_PHASE_LAST) && !(st->phase & EV_PHASE_FIRST)) {
-		dbg_puts("state_restore: ");
-		state_dbg(st);
-		dbg_puts(": called for last event!\n");
+
+	/*
+	 * don't restore last event of terminated frames
+	 */
+	if ((st->phase & EV_PHASE_LAST) && !(st->phase & EV_PHASE_FIRST))
 		return 0;
-	}
+
 	*rev = st->ev;
 	return 1;
 }
@@ -385,9 +317,6 @@ statelist_init(struct statelist *o)
 	o->first = NULL;
 	o->changed = 0;
 	o->serial = state_serial++;
-#ifdef STATE_PROF
-	prof_reset(&o->prof, "statlist/iter");
-#endif
 }
 
 /*
@@ -409,18 +338,15 @@ statelist_done(struct statelist *o)
 		 * the EV_CTL case is here for conv_xxx() functions
 		 */
 		if (!(i->phase & EV_PHASE_LAST) && i->ev.cmd != EV_CTL) {
-			dbg_puts("statelist_done: ");
-			ev_dbg(&i->ev);
-			dbg_puts(": unterminated frame\n");
+			log_puts("statelist_done: ");
+			ev_log(&i->ev);
+			log_puts(": unterminated frame\n");
 		}
 		inext = i->next;
 		statelist_rm(o, i);
 		state_del(i);
 	}
 
-#ifdef STATE_PROF
-	prof_dbg(&o->prof);
-#endif
 }
 
 void
@@ -428,10 +354,10 @@ statelist_dump(struct statelist *o)
 {
 	struct state *i;
 
-	dbg_puts("statelist_dump:\n");
+	log_puts("statelist_dump:\n");
 	for (i = o->first; i != NULL; i = i->next) {
-		ev_dbg(&i->ev);
-		dbg_puts("\n");
+		ev_log(&i->ev);
+		log_puts("\n");
 	}
 }
 
@@ -501,20 +427,11 @@ struct state *
 statelist_lookup(struct statelist *o, struct ev *ev)
 {
 	struct state *i;
-#ifdef STATE_PROF
-	unsigned time = 0;
-#endif
 	for (i = o->first; i != NULL; i = i->next) {
-#ifdef STATE_PROF
-		time++;
-#endif
 		if (state_match(i, ev)) {
 			break;
 		}
 	}
-#ifdef STATE_PROF
-	prof_val(&o->prof, time * 100);
-#endif
 	return i;
 }
 
@@ -531,111 +448,77 @@ statelist_update(struct statelist *statelist, struct ev *ev)
 {
 	struct state *st, *stnext;
 	unsigned phase;
-#ifdef STATE_PROF
-	unsigned time = 0;
-#endif
-	/*
-	 * we scan for a matching state, if it exists but is
-	 * terminated (phase = EV_PHASE_LAST) we purge it in order to
-	 * reuse the list entry. We cant just use statelist_lookup(),
-	 * because this will not work with nested frames (eg. if the
-	 * "top" state is purged but not the other one). So here we
-	 * inline a kind of 'lookup_for_write()' routine:
-	 */
+
+	phase = ev_phase(ev);
+
 	st = statelist->first;
 	for (;;) {
-#ifdef STATE_PROF
-		time++;
-#endif
 		if (st == NULL) {
 			st = state_new();
-			statelist_add(statelist, st);
 			st->flags = STATE_NEW;
+			statelist_add(statelist, st);
 			break;
 		}
+
 		stnext = st->next;
+
 		if (state_match(st, ev)) {
-			/*
-			 * found a matching state
-			 */
-			if (st->phase & EV_PHASE_LAST) {
-				/*
-				 * if the event is not tagged as
-				 * nested, we reached the deepest
-				 * state, so stop iterating here
-				 * else continue purging states
-				 */
-				if (!(st->flags & STATE_NESTED)) {
-					st->flags = 0;
-					break;
-				} else {
-					statelist_rm(statelist, st);
-					state_del(st);
-				}
-			} else {
+			if (!(st->phase == EV_PHASE_LAST) &&
+			    !(st->flags & STATE_BOGUS)) {
 				st->flags &= ~STATE_NEW;
 				break;
 			}
+
+			statelist_rm(statelist, st);
+			state_del(st);
 		}
 		st = stnext;
 	}
-#ifdef STATE_PROF
-	prof_val(&statelist->prof, time * 100);
-#endif
 
-	/*
-	 * if one of the following are true
-	 *	- there is no state because this is the first
-	 *	  event of a new frame, or this is a bogus next event
-	 *        (the beginning is missing)
-	 *	- there is a state, but this is for sure the
-	 *	  first event of a new frame (thus this is
-	 *	  de beginning of a nested frame)
-	 * then create a new state.
-	 */
-	phase = ev_phase(ev);
-	if (st->flags & STATE_NEW || st->phase & EV_PHASE_LAST) {
-		/*
-		 * this is new state or a terminated frame that we are
-		 * reusing
-		 */
-		if (!(phase & EV_PHASE_FIRST)) {
-			phase = EV_PHASE_FIRST | EV_PHASE_LAST;
-			st->flags |= STATE_BOGUS;
+	switch (phase) {
+	case EV_PHASE_FIRST:
+		if (st->flags != STATE_NEW) {
+			st = state_new();
+			st->flags = STATE_NEW | STATE_NESTED;
+			statelist_add(statelist, st);
 #ifdef STATE_DEBUG
-			dbg_puts("statelist_update: ");
-			ev_dbg(ev);
-			dbg_puts(": not first and no state\n");
+			log_puts("statelist_update: ");
+			ev_log(ev);
+			log_puts(": nested events, stacked\n");
 #endif
-		} else {
-			phase &= ~EV_PHASE_NEXT;
 		}
-	} else if (phase == EV_PHASE_FIRST) {
-		/*
-		 * this frame is not yet terminated. the incoming
-		 * event starts a new one that's conflicting
-		 */
+		break;
+	case EV_PHASE_NEXT:
+	case EV_PHASE_LAST:
+		if (st->flags == STATE_NEW) {
+			st->flags |= STATE_BOGUS;
+			phase |= EV_PHASE_FIRST;
+			phase &= ~EV_PHASE_NEXT;
 #ifdef STATE_DEBUG
-		dbg_puts("statelist_update: ");
-		ev_dbg(ev);
-		dbg_puts(": nested events, stacked\n");
+			log_puts("statelist_update: ");
+			ev_log(ev);
+			log_puts(": missing first event\n");
 #endif
-		st = state_new();
-		statelist_add(statelist, st);
-		st->flags = STATE_NESTED | STATE_NEW;
-	} else {
-		/*
-		 * this frame is not yet terminated, the incoming
-		 * event belongs to it
-		 */
-		phase &=  ~EV_PHASE_FIRST;
+		}
+		break;
+	case EV_PHASE_FIRST | EV_PHASE_NEXT:
+		phase &= (st->flags == STATE_NEW) ?
+		    ~EV_PHASE_NEXT : ~EV_PHASE_FIRST;
+		break;
+	case EV_PHASE_FIRST | EV_PHASE_LAST:
+		/* nothing */
+		break;
+	default:
+		log_puts("statelist_update: bad phase\n");
+		panic();
 	}
+
 	state_copyev(st, ev, phase);
 	statelist->changed = 1;
 #ifdef STATE_DEBUG
-	dbg_puts("statelist_update: updated: ");
-	state_dbg(st);
-	dbg_puts("\n");
+	log_puts("statelist_update: updated: ");
+	state_log(st);
+	log_puts("\n");
 #endif
 	return st;
 }
@@ -662,6 +545,11 @@ statelist_outdate(struct statelist *o)
 		 * states have both FIRST and LAST bits set
 		 */
 		if (i->phase == EV_PHASE_LAST) {
+#ifdef STATE_DEBUG
+			log_puts("statelist_outdate: ");
+			state_log(i);
+			log_puts(": removed\n");
+#endif
 			statelist_rm(o, i);
 			state_del(i);
 		} else {
