@@ -1,31 +1,17 @@
 /*
  * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -48,7 +34,7 @@
  *
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "timo.h"
 
 unsigned timo_debug = 0;
@@ -80,12 +66,12 @@ timo_add(struct timo *o, unsigned delta)
 
 #ifdef TIMO_DEBUG
 	if (o->set) {
-		dbg_puts("timo_add: already set\n");
-		dbg_panic();
+		log_puts("timo_add: already set\n");
+		panic();
 	}
 	if (delta == 0) {
-		dbg_puts("timo_add: zero timeout is evil\n");
-		dbg_panic();
+		log_puts("timo_add: zero timeout is evil\n");
+		panic();
 	}
 #endif
 	val = timo_abstime + delta;
@@ -117,7 +103,7 @@ timo_del(struct timo *o)
 		}
 	}
 	if (timo_debug)
-		dbg_puts("timo_del: not found\n");
+		log_puts("timo_del: not found\n");
 }
 
 /*
@@ -172,8 +158,8 @@ void
 timo_done(void)
 {
 	if (timo_queue != NULL) {
-		dbg_puts("timo_done: timo_queue not empty!\n");
-		dbg_panic();
+		log_puts("timo_done: timo_queue not empty!\n");
+		panic();
 	}
 	timo_queue = (struct timo *)0xdeadbeef;
 }
