@@ -1,31 +1,17 @@
 /*
  * Copyright (c) 2003-2010 Alexandre Ratchov <alex@caoua.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
  *
- * 	- Redistributions of source code must retain the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer.
- *
- * 	- Redistributions in binary form must reproduce the above
- * 	  copyright notice, this list of conditions and the
- * 	  following disclaimer in the documentation and/or other
- * 	  materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 /*
@@ -35,7 +21,7 @@
  * each function is described in the manual.html file
  */
 
-#include "dbg.h"
+#include "utils.h"
 #include "defs.h"
 #include "node.h"
 #include "exec.h"
@@ -43,7 +29,6 @@
 #include "cons.h"
 
 #include "textio.h"
-#include "lex.h"
 #include "parse.h"
 
 #include "mux.h"
@@ -60,7 +45,28 @@ struct song *usong;
 unsigned user_flag_batch = 0;
 unsigned user_flag_verb = 0;
 
-/* -------------------------------------------------- some tools --- */
+void
+exec_cb(struct exec *e, struct node *root)
+{
+	struct data *data;
+
+	if (root == NULL) {
+		log_puts("syntax error\n");
+		return;
+	}
+	if (e->result == RESULT_EXIT) {
+		log_puts("exitting, skiped\n");
+		return;
+	}
+	e->result = node_exec(root, e, &data);
+	if (data != NULL) {
+		if (data->type != DATA_NIL) {
+			data_print(data);
+			textout_putstr(tout, "\n");
+		}
+		data_delete(data);
+	}
+}
 
 /*
  * execute the script in the given file inside the 'exec' environment.
@@ -71,28 +77,30 @@ unsigned user_flag_verb = 0;
 unsigned
 exec_runfile(struct exec *exec, char *filename)
 {
-	struct parse *parse;
+	struct parse parse;
+	struct textin *in;
 	struct name **locals;
-	struct node *root;
-	struct data *data;
-	unsigned res;
+	int c;
 
-	res = 0;
-	root = NULL;
-	data = NULL;
-	parse = parse_new(filename);
-	if (!parse) {
+	in = textin_new(filename);
+	if (in == NULL)
 		return 0;
-	}
 	locals = exec->locals;
 	exec->locals = &exec->globals;
-	if (parse_prog(parse, &root)) {
-		res = node_exec(root, exec, &data);
+	parse_init(&parse, exec, exec_cb);
+	lex_init(&parse, filename, parse_cb, &parse);
+	for (;;) {
+		if (!textin_getchar(in, &c))
+			break;
+		lex_handle(&parse, c);
+		if (c < 0)
+			break;
 	}
 	exec->locals = locals;
-	parse_delete(parse);
-	node_delete(root);
-	return res;
+	textin_delete(in);
+	lex_done(&parse);
+	parse_done(&parse);
+	return exec->result == RESULT_OK || exec->result == RESULT_EXIT;
 }
 
 /*
@@ -129,10 +137,10 @@ exec_lookupchan_getnum(struct exec *o, char *var,
 
 	arg = exec_varlookup(o, var);
 	if (!arg) {
-		dbg_puts("exec_lookupchan_getnum: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupchan_getnum: no such var\n");
+		panic();
 	}
-	if (!data_list2chan(arg->data, dev, ch, input)) {
+	if (!data_getchan(arg->data, dev, ch, input)) {
 		return 0;
 	}
 	return 1;
@@ -143,7 +151,7 @@ exec_lookupchan_getnum(struct exec *o, char *var,
  * 'var'.  ('var' must be a reference)
  */
 unsigned
-exec_lookupchan_getref(struct exec *o, char *var, 
+exec_lookupchan_getref(struct exec *o, char *var,
     struct songchan **res, int input)
 {
 	struct var *arg;
@@ -151,8 +159,8 @@ exec_lookupchan_getref(struct exec *o, char *var,
 
 	arg = exec_varlookup(o, var);
 	if (!arg) {
-		dbg_puts("exec_lookupchan: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupchan: no such var\n");
+		panic();
 	}
 	if (arg->data->type == DATA_REF) {
 		i = song_chanlookup(usong, arg->data->val.ref, input);
@@ -177,6 +185,7 @@ exec_lookupfilt(struct exec *o, char *var, struct songfilt **res)
 {
 	char *name;
 	struct songfilt *f;
+
 	if (!exec_lookupname(o, var, &name)) {
 		return 0;
 	}
@@ -198,6 +207,7 @@ exec_lookupsx(struct exec *o, char *var, struct songsx **res)
 {
 	char *name;
 	struct songsx *t;
+
 	if (!exec_lookupname(o, var, &name)) {
 		return 0;
 	}
@@ -232,12 +242,12 @@ exec_lookupev(struct exec *o, char *name, struct ev *ev, int input)
 {
 	struct var *arg;
 	struct data *d;
-	unsigned dev, ch, max, num;
+	unsigned dev, ch, num;
 
 	arg = exec_varlookup(o, name);
 	if (!arg) {
-		dbg_puts("exec_lookupev: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupev: no such var\n");
+		panic();
 	}
 	d = arg->data;
 
@@ -248,20 +258,33 @@ exec_lookupev(struct exec *o, char *name, struct ev *ev, int input)
 	d = d->val.list;
 	if (!d || d->type != DATA_REF ||
 	    !ev_str2cmd(ev, d->val.ref) ||
-	    !EV_ISVOICE(ev)) {
+	    (!EV_ISVOICE(ev) && !EV_ISSX(ev))) {
 		cons_err("bad status in event spec");
 		return 0;
 	}
 	d = d->next;
 	if (!d) {
-		cons_err("no channel in event spec");
+		cons_err("channel and/or device missing in event spec");
 		return 0;
 	}
-	if (!data_list2chan(d, &dev, &ch, input)) {
-		return 0;
+	if (evinfo[ev->cmd].flags & EV_HAS_CH) {
+		if (!data_getchan(d, &dev, &ch, input)) {
+			return 0;
+		}
+		ev->dev = dev;
+		ev->ch = ch;
+	} else {
+		if (d->type != DATA_LONG) {
+			cons_err("device number expected in event spec");
+			return 0;
+		}
+		if (d->val.num < 0 || d->val.num >= DEFAULT_MAXNDEVS) {
+			cons_err("device number out of range in event spec");
+			return 0;
+		}
+		ev->dev = d->val.num;
+		ev->ch = 0;
 	}
-	ev->dev = dev;
-	ev->ch = ch;
 	d = d->next;
 	if (ev->cmd == EV_XCTL || ev->cmd == EV_CTL) {
 		if (!d || !data_getctl(d, &num)) {
@@ -269,33 +292,29 @@ exec_lookupev(struct exec *o, char *name, struct ev *ev, int input)
 		}
 		ev->ctl_num = num;
 	} else {
-		if (ev->cmd == EV_BEND || ev->cmd == EV_NRPN || ev->cmd == EV_RPN) {
-			max = EV_MAXFINE;
-		} else {
-			max = EV_MAXCOARSE;
-		}
-		if (!d || d->type != DATA_LONG || d->val.num < 0 || d->val.num > max) {
-			cons_err("bad byte0 in event spec");
+		if (ev->cmd == EV_XPC && d != NULL && d->type == DATA_NIL) {
+			ev->v0 = EV_UNDEF;
+		} else if (d == NULL ||
+		    d->type != DATA_LONG ||
+		    d->val.num < 0 ||
+		    d->val.num > evinfo[ev->cmd].v0_max) {
+			cons_err("bad v0 in event spec");
 			return 0;
-		}
-		ev->v0 = d->val.num;
+		} else
+			ev->v0 = d->val.num;
 	}
 	d = d->next;
-	if (ev->cmd == EV_PC || ev->cmd == EV_CAT || ev->cmd == EV_BEND) {
-		if (d) {
+	if (evinfo[ev->cmd].nparams < 2) {
+		if (d != NULL) {
 			cons_err("extra data in event spec");
 			return 0;
 		}
 	} else {
-		if (ev->cmd == EV_XPC || ev->cmd == EV_XCTL ||
-		    ev->cmd == EV_NRPN || ev->cmd == EV_RPN) {
-			max = EV_MAXFINE;
-		} else {
-			max = EV_MAXCOARSE;
-		}
-		if (!d || d->type != DATA_LONG ||
-		    d->val.num < 0 || d->val.num > max) {
-			cons_err("bad byte1 in event spec");
+		if (d == NULL ||
+		    d->type != DATA_LONG ||
+		    d->val.num < 0 ||
+		    d->val.num > evinfo[ev->cmd].v1_max) {
+			cons_err("bad v1 in event spec");
 			return 0;
 		}
 		ev->v1 = d->val.num;
@@ -310,6 +329,7 @@ exec_lookupev(struct exec *o, char *name, struct ev *ev, int input)
 	}
 	if (ev->cmd == EV_PC) {
 		ev->cmd = EV_XPC;
+		ev->pc_prog = ev->v0;
 		ev->pc_bank = EV_UNDEF;
 	}
 	return 1;
@@ -331,8 +351,8 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 
 	arg = exec_varlookup(o, name);
 	if (!arg) {
-		dbg_puts("exec_lookupev: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupev: no such var\n");
+		panic();
 	}
 	d = arg->data;
 	if (d->type != DATA_LIST) {
@@ -353,6 +373,10 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 		cons_err("bad status in event spec");
 		return 0;
 	}
+	if (!(evinfo[e->cmd].flags & EV_HAS_DEV))
+		e->dev_max = e->dev_min;
+	if (!(evinfo[e->cmd].flags & EV_HAS_CH))
+		e->ch_max = e->ch_min;
 	e->v0_min = evinfo[e->cmd].v0_min;
 	e->v0_max = evinfo[e->cmd].v0_max;
 	e->v1_min = evinfo[e->cmd].v1_min;
@@ -377,17 +401,19 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 		e->dev_min = e->dev_max = i->dev;
 		e->ch_min = e->ch_max = i->ch;
 	} else if (d->type == DATA_LIST) {
-		if (!d->val.list) {		/* empty list = any chan/dev */
-			/* nothing */
+		if (!d->val.list) {
+			/* empty list = any chan/dev */
 		} else if (d->val.list &&
 		    d->val.list->next &&
 		    !d->val.list->next->next) {
-			if (!data_list2range(d->val.list, 0, EV_MAXDEV, &lo, &hi)) {
+			if (!data_getrange(d->val.list, 0,
+				EV_MAXDEV, &lo, &hi)) {
 				return 0;
 			}
 			e->dev_min = lo;
 			e->dev_max = hi;
-			if (!data_list2range(d->val.list->next, 0, EV_MAXCH, &lo, &hi)) {
+			if (!data_getrange(d->val.list->next, 0,
+				EV_MAXCH, &lo, &hi)) {
 				return 0;
 			}
 			e->ch_min = lo;
@@ -403,8 +429,9 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 		}
 		e->dev_min = e->dev_max = d->val.num;
 		e->ch_min = 0;
-		e->ch_max = EV_MAXCH;
-	} else { 
+		e->ch_max = (evinfo[e->cmd].flags & EV_HAS_CH) ?
+			EV_MAXCH : e->ch_min;
+	} else {
 		cons_err("list or ref expected as channel range spec");
 		return 0;
 	}
@@ -425,8 +452,10 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 			return 0;
 		}
 		e->v0_min = e->v0_max = hi;
+	} else if (e->cmd == EV_XPC && d->type == DATA_NIL) {
+		e->v0_min = e->v0_max = EV_UNDEF;
 	} else {
-		if (!data_list2range(d, e->v0_min, e->v0_max, &min, &max)) {
+		if (!data_getrange(d, e->v0_min, e->v0_max, &min, &max)) {
 			return 0;
 		}
 		e->v0_min = min;
@@ -444,7 +473,7 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 	if (evinfo[e->cmd].nranges < 2) {
 		goto toomany;
 	}
-	if (!data_list2range(d, e->v1_min, e->v1_max, &min, &max)) {
+	if (!data_getrange(d, e->v1_min, e->v1_max, &min, &max)) {
 		return 0;
 	}
 	e->v1_min = min;
@@ -459,8 +488,9 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 	 */
 	if (e->cmd == EVSPEC_PC) {
 		e->cmd = EVSPEC_XPC;
-		e->v1_min = evinfo[e->cmd].v1_min;
-		e->v1_max = evinfo[e->cmd].v1_max;
+		e->v1_min = e->v0_min;
+		e->v1_max = e->v0_max;
+		e->v0_min = e->v0_max = EV_UNDEF;
 	} else if (e->cmd == EVSPEC_CTL) {
 		e->cmd = EVSPEC_XCTL;
 		e->v1_min =  (e->v1_min << 7) & 0x3fff;
@@ -468,9 +498,9 @@ exec_lookupevspec(struct exec *o, char *name, struct evspec *e, int input)
 	}
 
 #if 0
-	dbg_puts("lookupevspec: ");
-	evspec_dbg(e);
-	dbg_puts("\n");
+	log_puts("lookupevspec: ");
+	evspec_log(e);
+	log_puts("\n");
 #endif
 
 	return 1;
@@ -492,8 +522,8 @@ exec_lookupctl(struct exec *o, char *var, unsigned *num)
 
 	arg = exec_varlookup(o, var);
 	if (!arg) {
-		dbg_puts("exec_lookupctl: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupctl: no such var\n");
+		panic();
 	}
 	return data_getctl(arg->data, num);
 }
@@ -542,39 +572,11 @@ data_print(struct data *d)
 		textout_putlong(tout, d->val.range.max);
 		break;
 	default:
-		dbg_puts("data_print: unknown type\n");
+		log_puts("data_print: unknown type\n");
 		break;
 	}
 }
 
-/*
- * convert 2 integer lists to channels
- */
-unsigned
-data_num2chan(struct data *o, unsigned *res_dev, unsigned *res_ch)
-{
-	long dev, ch;
-
-	if (o == NULL ||
-	    o->next == NULL ||
-	    o->next->next != NULL ||
-	    o->type != DATA_LONG ||
-	    o->next->type != DATA_LONG) {
-		cons_err("bad {dev midichan} in spec");
-		return 0;
-	}
-	dev = o->val.num;
-	ch = o->next->val.num;
-	if (ch < 0 || ch > EV_MAXCH ||
-	    dev < 0 || dev > EV_MAXDEV) {
-		cons_err("bad dev/midichan ranges");
-		return 0;
-	}
-	*res_dev = dev;
-	*res_ch = ch;
-	return 1;
-}
-
 /*
  * lookup the value of the given controller
  */
@@ -586,8 +588,8 @@ exec_lookupval(struct exec *o, char *n, unsigned isfine, unsigned *r)
 
 	arg = exec_varlookup(o, n);
 	if (!arg) {
-		dbg_puts("exec_lookupval: no such var\n");
-		dbg_panic();
+		log_puts("exec_lookupval: no such var\n");
+		panic();
 	}
 	if (arg->data->type == DATA_NIL) {
 		*r = EV_UNDEF;
@@ -612,12 +614,31 @@ exec_lookupval(struct exec *o, char *n, unsigned isfine, unsigned *r)
  *	- a pair of integers '{ dev midichan }'
  */
 unsigned
-data_list2chan(struct data *o, unsigned *res_dev, unsigned *res_ch, int input)
+data_getchan(struct data *o, unsigned *res_dev, unsigned *res_ch, int input)
 {
 	struct songchan *i;
+	long dev, ch;
 
 	if (o->type == DATA_LIST) {
-		return data_num2chan(o->val.list, res_dev, res_ch);
+		o = o->val.list;
+		if (o == NULL ||
+		    o->next == NULL ||
+		    o->next->next != NULL ||
+		    o->type != DATA_LONG ||
+		    o->next->type != DATA_LONG) {
+			cons_err("bad {dev midichan} pair");
+			return 0;
+		}
+		dev = o->val.num;
+		ch = o->next->val.num;
+		if (ch < 0 || ch > EV_MAXCH ||
+		    dev < 0 || dev > EV_MAXDEV) {
+			cons_err("bad dev/midichan ranges");
+			return 0;
+		}
+		*res_dev = dev;
+		*res_ch = ch;
+		return 1;
 	} else if (o->type == DATA_REF) {
 		i = song_chanlookup(usong, o->val.ref, input);
 		if (i == NULL) {
@@ -640,15 +661,18 @@ data_list2chan(struct data *o, unsigned *res_dev, unsigned *res_ch, int input)
  *	- a single integer (then min = max)
  */
 unsigned
-data_list2range(struct data *d, unsigned min, unsigned max,
-    unsigned *lo, unsigned *hi) {
+data_getrange(struct data *d, unsigned min, unsigned max,
+    unsigned *rlo, unsigned *rhi)
+{
+	long lo, hi;
+
     	if (d->type == DATA_LONG) {
-		*lo = *hi = d->val.num;
+		lo = hi = d->val.num;
 	} else if (d->type == DATA_LIST) {
 		d = d->val.list;
 		if (!d) {
-			*lo = min;
-			*hi = max;
+			*rlo = min;
+			*rhi = max;
 			return 1;
 		}
 		if (!d->next || d->next->next ||
@@ -656,19 +680,21 @@ data_list2range(struct data *d, unsigned min, unsigned max,
 			cons_err("exactly 0 or 2 numbers expected in range spec");
 			return 0;
 		}
-		*lo = d->val.num;
-		*hi = d->next->val.num;
+		lo = d->val.num;
+		hi = d->next->val.num;
 	} else if (d->type == DATA_RANGE) {
-		*lo = d->val.range.min;
-		*hi = d->val.range.max;
+		lo = d->val.range.min;
+		hi = d->val.range.max;
 	} else {
 		cons_err("range or number expected in range spec");
 		return 0;
 	}
-	if (*lo < min || *lo > max || *hi < min || *hi > max || *lo > *hi) {
+	if (lo < min || lo > max || hi < min || hi > max || lo > hi) {
 		cons_err("range values out of bounds");
 		return 0;
 	}
+	*rlo = lo;
+	*rhi = hi;
 	return 1;
 }
 
@@ -676,7 +702,7 @@ data_list2range(struct data *d, unsigned min, unsigned max,
  * convert a list to bitmap of continuous controllers
  */
 unsigned
-data_list2ctlset(struct data *d, unsigned *res)
+data_getctlset(struct data *d, unsigned *res)
 {
 	unsigned ctlset;
 
@@ -690,10 +716,6 @@ data_list2ctlset(struct data *d, unsigned *res)
 			cons_err("controller number out of range 0..31");
 			return 0;
 		}
-		if (evctl_isreserved(d->val.num)) {
-			cons_erru(d->val.num, "controller number reserved");
-			return 0;
-		}
 		ctlset |= (1 << d->val.num);
 		d = d->next;
 	}
@@ -701,6 +723,37 @@ data_list2ctlset(struct data *d, unsigned *res)
 	return 1;
 }
 
+/*
+ * convert a list to bitmap of CONV_xxx constants
+ */
+unsigned
+data_getxev(struct data *d, unsigned *res)
+{
+	static unsigned cmds[] = {EV_XPC, EV_NRPN, EV_RPN};
+	unsigned i, conv, cmd;
+
+	conv = 0;
+	while (d) {
+		if (d->type != DATA_REF) {
+		err:
+			cons_err("xpc, rpn, or nrpn expected as flag");
+			return 0;
+		}
+		i = 0;
+		for (;;) {
+			if (i == sizeof(cmds) / sizeof(int))
+				goto err;
+			cmd = cmds[i];
+			if (str_eq(d->val.ref, evinfo[cmd].ev))
+				break;
+			i++;
+		}
+		conv |= (1 << cmd);
+		d = d->next;
+	}
+	*res = conv;
+	return 1;
+}
 
 /*
  * check if the pattern in data (list of integers)
@@ -751,10 +804,6 @@ data_getctl(struct data *d, unsigned *num)
 			cons_err("7bit ctl number out of bounds");
 			return 0;
 		}
-		if (evctl_isreserved(d->val.num)) {
-			cons_err("controller is reserved for bank, rpn, nrpn");
-			return 0;
-		}
 		*num = d->val.num;
 	} else if (d->type == DATA_REF) {
 		if (!evctl_lookup(d->val.ref, num)) {
@@ -768,105 +817,128 @@ data_getctl(struct data *d, unsigned *num)
 	return 1;
 }
 
-/* ---------------------------------------- interpreter functions --- */
+static struct parse parse;
+static struct exec *exec;
+static unsigned exitcode;
+int done = 0;
 
-
-unsigned
-user_func_info(struct exec *o, struct data **r)
+void
+user_onchar(void *arg, int c)
 {
-	exec_dumpprocs(o);
-	exec_dumpvars(o);
-	return 1;
+	if (done)
+		return;
+	lex_handle(&parse, c);
+	if (c < 0) {
+		/* end-of-file (user quit) */
+		exitcode = 1;
+		done = 1;
+	}
+	if (exec->result == RESULT_EXIT) {
+		exitcode = 1;
+		done = 1;
+	}
+	if (exec->result == RESULT_ERR && user_flag_batch) {
+		exitcode = 0;
+		done = 1;
+	}
+	if (c == '\n')
+		cons_ready();
 }
 
-
-unsigned
-user_func_shut(struct exec *o, struct data **r)
+void
+user_oncompl(void *arg, char *text, int curs, int used, int *rstart, int *rend)
 {
-	unsigned i;
-	struct ev ev;
-	struct mididev *dev;
+	struct name *n;
+	int start, end;
+	int c, quote;
 
-	/*
-	 * XXX: should raise mode to SONG_IDLE and
-	 * use mixout
-	 */
-	if (!song_try_mode(usong, 0)) {
-		return 0;
+	quote = 0;
+	start = curs;
+	while (1) {
+		if (start-- == 0)
+			break;
+		if (text[start] == '"')
+			quote ^= 1;
 	}
-	mux_open();
-	for (dev = mididev_list; dev != NULL; dev = dev->next) {
-		for (i = 0; i < EV_MAXCH; i++) {
-			ev.cmd = EV_XCTL;
-			ev.dev = dev->unit;
-			ev.ch = i;
-			ev.ctl_num = 121;
-			ev.ctl_val = 0;
-			mux_putev(&ev);
-			ev.cmd = EV_XCTL;
-			ev.dev = dev->unit;
-			ev.ch = i;
-			ev.ctl_num = 123;
-			ev.ctl_val = 0;
-			mux_putev(&ev);
-			ev.cmd = EV_BEND;
-			ev.dev = dev->unit;
-			ev.ch = i;
-			ev.bend_val = EV_BEND_DEFAULT;
-			mux_putev(&ev);
+
+	if (quote) {
+
+		/*
+		 * quoted test is file-name, find directory components
+		 * and scan directory
+		 */
+
+		end = curs;
+		while (1) {
+			if (end == used || text[end] == '"')
+				break;
+			end++;
 		}
-	}
-	mux_close();
-	return 1;
-}
 
-unsigned
-user_func_proclist(struct exec *o, struct data **r)
-{
-	struct proc *i;
-	struct data *d, *n;
-
-	d = data_newlist(NULL);
-	PROC_FOREACH(i, o->procs) {
-		if (i->code->vmt == &node_vmt_slist) {
-			n = data_newref(i->name.str);
-			data_listadd(d, n);
+		start = curs;
+		while (1) {
+			if (text[start - 1] == '"')
+				break;
+			start--;
 		}
-	}
-	*r = d;
-	return 1;
-}
 
-unsigned
-user_func_builtinlist(struct exec *o, struct data **r)
-{
-	struct proc *i;
-	struct data *d, *n;
+		user_oncompl_path(text, &start, &end);
+	} else {
 
-	d = data_newlist(NULL);
-	PROC_FOREACH(i, o->procs) {
-		if (i->code->vmt == &node_vmt_builtin) {
-			n = data_newref(i->name.str);
-			data_listadd(d, n);
+		/*
+		 * unquoted text is a proc name; complete
+		 * if it's either at the beginning or after '['
+		 */
+
+		start = curs;
+		while (1) {
+			if (start == 0)
+				break;
+			c = text[start - 1];
+			if (!IS_IDNEXT(c)) {
+				if (c == '[')
+					break;
+				return;
+			}
+			start--;
 		}
+		end = curs;
+		while (1) {
+			if (end == used || !IS_IDNEXT(text[end]))
+				break;
+			end++;
+		}
+
+		for (n = exec->procs; n != NULL; n = n->next)
+			el_compladd(n->str);
 	}
-	*r = d;
-	return 1;
+
+	*rstart = start;
+	*rend = end;
 }
 
+struct el_ops user_el_ops = {
+	user_onchar,
+	user_oncompl
+};
+
 unsigned
 user_mainloop(void)
 {
-	struct parse *parse;
-	struct exec *exec;
-	struct node *root;
-	struct data *data;
-	unsigned result, exitcode;
+	cons_init(&user_el_ops, NULL);
+	textio_init();
+	evctl_init();
+	seqev_pool_init(DEFAULT_MAXNSEQEVS);
+	state_pool_init(DEFAULT_MAXNSTATES);
+	chunk_pool_init(DEFAULT_MAXNCHUNKS);
+	sysex_pool_init(DEFAULT_MAXNSYSEXS);
+	seqptr_pool_init(DEFAULT_MAXNSEQPTRS);
 
 	/*
 	 * create the project (ie the song) and
 	 * the execution environment of the interpreter
 	 */
+	mididev_listinit();
 	usong = song_new();
 	exec = exec_new();
 
@@ -874,18 +946,19 @@ user_mainloop(void)
 	 * register built-in functions
 	 */
 	exec_newbuiltin(exec, "print", blt_print,
-			name_newarg("value", NULL));
+			name_newarg("...", NULL));
 	exec_newbuiltin(exec, "err", blt_err,
 			name_newarg("message", NULL));
 	exec_newbuiltin(exec, "h", blt_h,
-			name_newarg("function", NULL));
+			name_newarg("...", NULL));
 	exec_newbuiltin(exec, "exec", blt_exec,
 			name_newarg("filename", NULL));
 	exec_newbuiltin(exec, "debug", blt_debug,
 			name_newarg("flag",
 			name_newarg("value", NULL)));
+	exec_newbuiltin(exec, "version", blt_version, NULL);
 	exec_newbuiltin(exec, "panic", blt_panic, NULL);
-	exec_newbuiltin(exec, "info", user_func_info, NULL);
+	exec_newbuiltin(exec, "info", blt_info, NULL);
 
 	exec_newbuiltin(exec, "getunit", blt_getunit, NULL);
 	exec_newbuiltin(exec, "setunit", blt_setunit,
@@ -899,6 +972,8 @@ user_mainloop(void)
 	exec_newbuiltin(exec, "getlen", blt_getlen, NULL);
 	exec_newbuiltin(exec, "sel", blt_sel,
 			name_newarg("length", NULL));
+	exec_newbuiltin(exec, "loop", blt_loop, NULL);
+	exec_newbuiltin(exec, "noloop", blt_noloop, NULL);
 	exec_newbuiltin(exec, "getq", blt_getq, NULL);
 	exec_newbuiltin(exec, "setq", blt_setq,
 			name_newarg("step", NULL));
@@ -962,12 +1037,21 @@ user_mainloop(void)
 	exec_newbuiltin(exec, "ctlunconf", blt_ctlunconf,
 			name_newarg("name", NULL));
 	exec_newbuiltin(exec, "ctlinfo", blt_ctlinfo, NULL);
+	exec_newbuiltin(exec, "evpat", blt_evpat,
+			name_newarg("name",
+			name_newarg("pattern", NULL)));
+	exec_newbuiltin(exec, "evinfo", blt_evinfo, NULL);
 	exec_newbuiltin(exec, "m", blt_metro,
 			name_newarg("onoff", NULL));
 	exec_newbuiltin(exec, "metrocf", blt_metrocf,
 			name_newarg("eventhi",
 			name_newarg("eventlo", NULL)));
-
+	exec_newbuiltin(exec, "tap", blt_tap,
+			name_newarg("mode", NULL));
+	exec_newbuiltin(exec, "tapev", blt_tapev,
+			name_newarg("evspec", NULL));
+	exec_newbuiltin(exec, "u", blt_undo, NULL);
+	exec_newbuiltin(exec, "ul", blt_undolist, NULL);
 	exec_newbuiltin(exec, "tlist", blt_tlist, NULL);
 	exec_newbuiltin(exec, "tnew", blt_tnew,
 			name_newarg("trackname", NULL));
@@ -985,6 +1069,7 @@ user_mainloop(void)
 			name_newarg("filtname", NULL));
 	exec_newbuiltin(exec, "tgetf", blt_tgetf, NULL);
 	exec_newbuiltin(exec, "tcheck", blt_tcheck, NULL);
+	exec_newbuiltin(exec, "trewrite", blt_trewrite, NULL);
 	exec_newbuiltin(exec, "tcut", blt_tcut, NULL);
 	exec_newbuiltin(exec, "tclr", blt_tclr, NULL);
 	exec_newbuiltin(exec, "tpaste", blt_tpaste, NULL);
@@ -993,15 +1078,20 @@ user_mainloop(void)
 			name_newarg("amount", NULL));
 	exec_newbuiltin(exec, "tmerge", blt_tmerge,
 			name_newarg("source", NULL));
-	exec_newbuiltin(exec, "tquant", blt_tquant,
+	exec_newbuiltin(exec, "tquanta", blt_tquanta,
+			name_newarg("rate", NULL));
+	exec_newbuiltin(exec, "tquantf", blt_tquantf,
 			name_newarg("rate", NULL));
 	exec_newbuiltin(exec, "ttransp", blt_ttransp,
 			name_newarg("halftones", NULL));
+	exec_newbuiltin(exec, "tvcurve", blt_tvcurve,
+			name_newarg("weight", NULL));
 	exec_newbuiltin(exec, "tevmap", blt_tevmap,
 			name_newarg("from",
 			name_newarg("to", NULL)));
 	exec_newbuiltin(exec, "tclist", blt_tclist, NULL);
 	exec_newbuiltin(exec, "tinfo", blt_tinfo, NULL);
+	exec_newbuiltin(exec, "tdump", blt_tdump, NULL);
 
 	exec_newbuiltin(exec, "ilist", blt_ilist, NULL);
 	exec_newbuiltin(exec, "iexists", blt_iexists,
@@ -1093,10 +1183,15 @@ user_mainloop(void)
 	exec_newbuiltin(exec, "xadd", blt_xadd,
 			name_newarg("devnum",
 			name_newarg("data", NULL)));
+	exec_newbuiltin(exec, "ximport", blt_ximport,
+		        name_newarg("devnum",
+			name_newarg("path", NULL)));
+	exec_newbuiltin(exec, "xexport", blt_xexport,
+			name_newarg("path", NULL));
 
-	exec_newbuiltin(exec, "shut", user_func_shut, NULL);
-	exec_newbuiltin(exec, "proclist", user_func_proclist, NULL);
-	exec_newbuiltin(exec, "builtinlist", user_func_builtinlist, NULL);
+	exec_newbuiltin(exec, "shut", blt_shut, NULL);
+	exec_newbuiltin(exec, "proclist", blt_proclist, NULL);
+	exec_newbuiltin(exec, "builtinlist", blt_builtinlist, NULL);
 
 	exec_newbuiltin(exec, "dnew", blt_dnew,
 			name_newarg("devnum",
@@ -1104,6 +1199,7 @@ user_mainloop(void)
 			name_newarg("mode", NULL))));
 	exec_newbuiltin(exec, "ddel", blt_ddel,
 			name_newarg("devnum", NULL));
+	exec_newbuiltin(exec, "dlist", blt_dlist, NULL);
 	exec_newbuiltin(exec, "dmtcrx", blt_dmtcrx,
 			name_newarg("devnum", NULL));
 	exec_newbuiltin(exec, "dmmctx", blt_dmmctx,
@@ -1123,6 +1219,12 @@ user_mainloop(void)
 	exec_newbuiltin(exec, "doxctl", blt_doxctl,
 			name_newarg("devnum",
 			name_newarg("ctlset", NULL)));
+	exec_newbuiltin(exec, "diev", blt_diev,
+			name_newarg("devnum",
+			name_newarg("flags", NULL)));
+	exec_newbuiltin(exec, "doev", blt_doev,
+			name_newarg("devnum",
+			name_newarg("flags", NULL)));
 
 	/*
 	 * run the user startup script: $HOME/.midishrc or /etc/midishrc
@@ -1136,68 +1238,29 @@ user_mainloop(void)
 	/*
 	 * create the parser and start parsing standard input
 	 */
-	parse = parse_new(NULL);
-	if (parse == NULL) {
-		return 0;
-	}
+	parse_init(&parse, exec, exec_cb);
+	lex_init(&parse, "stdin", parse_cb, &parse);
 
+	cons_ready();
 	cons_putpos(usong->curpos, 0, 0);
 
-	root = NULL;
-	data = NULL;
-	for (;;) {
-		/*
-		 * print mem_alloc() and mem_free() stats, useful to
-		 * track memory leaks
-		 */
-		mem_stats();
+	done = 0;
+	while (!done && mux_mdep_wait(1))
+		; /* nothing */
 
-		/*
-		 * parse a block
-		 */
-		if (!parse_getsym(parse)) {
-			goto err;
-		}
-		if (parse->lex.id == TOK_EOF) {
-			/* end-of-file (user quit) */
-			exitcode = 1;
-			break;
-		}
-		parse_ungetsym(parse);
-		if (!parse_line(parse, &root)) {
-			node_delete(root);
-			root = NULL;
-			goto err;
-		}
-
-		/*
-		 * at this stage no parse error, execute the tree
-		 */
-		result = node_exec(root, exec, &data);
-		node_delete(root);
-		root = NULL;
-		if (result == RESULT_OK) {
-			continue;
-		}
-		if (result == RESULT_EXIT) {
-			exitcode = 1;	/* 1 means success */
-			break;
-		}
-
-	err:
-		/*
-		 * in batch mode stop on the first error
-		 */
-		if (user_flag_batch) {
-			exitcode = 0;	/* 0 means failure */
-			break;
-		}
-	}
-
-	parse_delete(parse);
+	lex_done(&parse);
+	parse_done(&parse);
 	exec_delete(exec);
 	song_delete(usong);
 	usong = NULL;
 	mididev_listdone();
+	seqptr_pool_done();
+	sysex_pool_done();
+	chunk_pool_done();
+	state_pool_done();
+	seqev_pool_done();
+	evctl_done();
+	textio_done();
+	cons_done();
 	return exitcode;
 }
